# -*- coding: utf-8 -*-
###
# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import Resource


class StorageVolumeTemplates(Resource):
    """
    Storage Volume Templates API client.

    """
    URI = '/rest/storage-volume-templates'

    DEFAULT_VALUES = {
        '200': {"type": "StorageVolumeTemplateV3"},
        '300': {"type": "StorageVolumeTemplateV3"}
    }

    def __init__(self, connection, data=None):
        super(StorageVolumeTemplates, self).__init__(connection, data)

    def get_connectable_volume_templates(self, start=0, count=-1, filter='', query='', sort=''):
        """
        Gets the storage volume templates that are available on the specified networks based on the storage system
        port's expected network connectivity. If there are no storage volume templates that meet the specified
        connectivity criteria, an empty collection will be returned.

        Returns:
            list: Storage volume templates.
        """
        uri = self.URI + "/connectable-volume-templates"

        get_uri = self._helper.build_query_uri(start=start, count=count, filter=filter,
                                               query=query, sort=sort, uri=uri)
        return self._helper.do_get(get_uri)

    def get_reachable_volume_templates(self, start=0, count=-1, filter='', query='', sort='',
                                       networks=None, scope_uris='', private_allowed_only=False):
        """
        Gets the storage templates that are connected on the specified networks based on the storage system
        port's expected network connectivity.

        Returns:
            list: Storage volume templates.
        """
        uri = self.URI + "/reachable-volume-templates"

        uri += "?networks={}&privateAllowedOnly={}".format(networks, private_allowed_only)

        get_uri = self._helper.build_query_uri(start=start, count=count, filter=filter,
                                               query=query, sort=sort, uri=uri, scope_uris=scope_uris)
        return self._helper.do_get(get_uri)

    def get_compatible_systems(self):
        """
        Retrieves a collection of all storage systems that is applicable to this storage volume template.

        Returns:
            list: Storage systems.
        """
        uri = "{}/compatible-systems".format(self.data["uri"])
        return self._helper.do_get(uri)

    def delete(self, force=False, timeout=-1):
        """
        Deletes the specified storage volume template.

        Args:
            force (bool):
                 If set to true, the operation completes despite any problems with
                 network connectivity or errors on the resource itself. The default is false.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
        Returns:
            bool: Indicates if the resource was successfully deleted.
        """
        custom_headers = {'Accept-Language': 'en_US', 'If-Match': '*'}
        uri = self.data['uri']

        return self._helper.delete(uri, force=force, timeout=timeout, custom_headers=custom_headers)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from future import standard_library

standard_library.install_aliases()

from hpeOneView.resources.resource import Resource, ResourceHelper, ResourcePatchMixin


class NetworkSets(Resource, ResourcePatchMixin):
    """
    Network Sets API client.

    """
    URI = '/rest/network-sets'

    DEFAULT_VALUES = {
        '200': {"type": "network-set"},
        '300': {"type": "network-setV300"},
        '500': {"type": "network-setV300"},
        '600': {"type": "network-setV4"},
        '800': {"type": "network-setV4"},
        '1000': {"type": "network-setV4"},
        '1200': {"type": "network-setV5"},
        '1600': {"type": "network-setV5"},
        '1800': {"type": "network-setV5"}
    }

    def __init__(self, connection, data=None):
        super(NetworkSets, self).__init__(connection, data)

    def get_all_without_ethernet(self, start=0, count=-1, filter='', sort=''):
        """
        Gets a paginated collection of network sets without ethernet. The collection is based
        on optional sorting and filtering and is constrained by start and count parameters.

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items.
                The actual number of items in the response might differ from the requested
                count if the sum of start and count exceeds the total number of items.
            filter (list or str):
                A general filter/query string to narrow the list of items returned. The
                default is no filter; all resources are returned.
            sort:
                The sort order of the returned data set. By default, the sort order is based
                on create time with the oldest entry first.

        Returns:
            list: List of network sets, excluding Ethernet networks.
        """
        without_ethernet_client = ResourceHelper("/rest/network-sets/withoutEthernet",
                                                 self._connection,
                                                 self._task_monitor)

        return without_ethernet_client.get_all(start, count, filter=filter, sort=sort)

    def get_without_ethernet(self):
        """
        Gets the network set with the specified ID or URI without ethernet.

        Returns:
            dict: Network set excluding Ethernet networks.
         uri = "{}/script".format(self.data['uri'])
        return self._helper.do_get(uri)
        """
        uri = "{}/withoutEthernet".format(self.data['uri'])
        return self._helper.do_get(uri)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()

from hpeOneView.resources.resource import ResourceClient


class Versions(object):
    """
    Version API client. It indicates the range of API versions supported by the appliance.

    """
    URI = '/rest/version'

    def __init__(self, con):
        self._client = ResourceClient(con, self.URI)

    def get_version(self):
        """
        Returns the range of possible API versions supported by the appliance.
        The response contains the current version and the minimum version.
        The current version is the recommended version to specify in the REST header.
        The other versions are supported for backward compatibility, but might not support the most current features.

        Returns:
            dict: The minimum and maximum supported API versions.
        """
        version = self._client.get(self.URI)
        return version



# -*- coding: utf-8 -*-
###
# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import ResourceClient


class CertificateRabbitMQ(object):
    URI = '/rest/certificates/client/rabbitmq'

    def __init__(self, con):
        self._connection = con
        self._client = ResourceClient(con, self.URI)

    def generate(self, information, timeout=-1):
        """
        Generates a self signed certificate or an internal CA signed certificate for RabbitMQ clients.

        Args:
            information (dict): Information to generate the certificate for RabbitMQ clients.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: RabbitMQ certificate generated
        """
        return self._client.create(information, timeout=timeout)

    def get(self, alias_name):
        """
        Retrieves the base-64 encoded certificate associated with the RabbitMQ user.

        Args:
            alias_name: Key pair associated with the RabbitMQ

        Returns:
            dict: RabbitMQ certificate
        """
        return self._client.get(alias_name)

    def get_key_pair(self, alias_name):
        """
        Retrieves the public and private key pair associated with the specified alias name.

        Args:
            alias_name: Key pair associated with the RabbitMQ

        Returns:
            dict: RabbitMQ certificate
        """
        uri = self.URI + "/keypair/" + alias_name
        return self._client.get(uri)

    def get_keys(self, alias_name, key_format):
        """
        Retrieves the contents of PKCS12 file in the format specified.
        This PKCS12 formatted file contains both the certificate as well as the key file data.
        Valid key formats are Base64 and PKCS12.

        Args:
            alias_name: Key pair associated with the RabbitMQ
            key_format: Valid key formats are Base64 and PKCS12.
        Returns:
            dict: RabbitMQ certificate
        """
        uri = self.URI + "/keys/" + alias_name + "?format=" + key_format
        return self._client.get(uri)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import Resource, ensure_resource_client, unavailable_method


class ManagedSANs(Resource):
    """
    Managed SANs API client.

    """
    URI = '/rest/fc-sans/managed-sans'

    def __init__(self, connection, data=None):
        super(ManagedSANs, self).__init__(connection, data)

    def get_all(self, start=0, count=-1, query='', sort=''):
        """
        Retrieves the list of registered Managed SANs

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items. The actual number of items in
                the response may differ from the requested count if the sum of start and count exceed the total number
                of items.
            query:
                A general query string to narrow the list of resources returned.
                The default is no query - all resources are returned.
            sort:
                The sort order of the returned data set. By default, the sort order is based
                on create time with the oldest entry first.

        Returns:
            list: A list of Managed SANs
        """
        return self._helper.get_all(start=start, count=count, query=query, sort=sort)

    def get_by_name(self, name):
        """
        Gets a Managed SAN by name.

        Args:
            name: Name of the Managed SAN

        Returns:
            dict: Managed SAN.
        """
        managed_sans = self.get_all()
        result = [x for x in managed_sans if x['name'] == name]

        resource = result[0] if result else None
        if resource:
            resource = self.new(self._connection, resource)

        return resource

    def create(self):
        """Create method is not available"""
        unavailable_method()

    def delete(self):
        """Delete method is not available"""
        unavailable_method()

    @ensure_resource_client
    def get_endpoints(self, start=0, count=-1, filter='', sort=''):
        """
        Gets a list of endpoints in a SAN.

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items.
                The actual number of items in the response might differ from the requested
                count if the sum of start and count exceeds the total number of items.
            filter (list or str):
                A general filter/query string to narrow the list of items returned. The
                default is no filter; all resources are returned.
            sort:
                The sort order of the returned data set. By default, the sort order is based
                on create time with the oldest entry first.

        Returns:
            list: A list of endpoints.
        """
        uri = "{}/endpoints/".format(self.data["uri"])
        return self._helper.get_all(start, count, filter=filter, sort=sort, uri=uri)

    @ensure_resource_client
    def create_endpoints_csv_file(self, timeout=-1):
        """
        Creates an endpoints CSV file for a SAN.

        Args:
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation in
                OneView, just stops waiting for its completion.

        Returns:
            dict: Endpoint CSV File Response.
        """
        uri = "{}/endpoints/".format(self.data["uri"])
        return self._helper.do_post(uri, {}, timeout, None)

    @ensure_resource_client
    def create_issues_report(self, timeout=-1):
        """
        Creates an unexpected zoning report for a SAN.

        Args:
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation in
                OneView, just stops waiting for its completion.

        Returns:
            list: A list of FCIssueResponse dict.
        """
        uri = "{}/issues/".format(self.data["uri"])
        return self._helper.create_report(uri, timeout)

    def get_wwn(self, wwn):
        """
        Retrieves a list of associations between provided WWNs and the SANs (if any) on which they reside.

        Note:
            This method is available for API version 300 or later.

        Args:
            wwn (str): The WWN that may be associated with the SAN.

        Returns:
            list: Associations between provided WWNs and the SANs
        """
        uri = '/rest/fc-sans/managed-sans?locate=' + wwn
        return self._helper.do_get(uri)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import Resource, ResourcePatchMixin, ensure_resource_client


class LogicalEnclosures(ResourcePatchMixin, Resource):
    """
    The logical enclosure resource provides methods for managing one or more enclosures that are
    linked or stacked with stacking links.

    """
    URI = '/rest/logical-enclosures'

    def __init__(self, connection, data=None):
        super(LogicalEnclosures, self).__init__(connection, data)

    def get_all(self, start=0, count=-1, filter='', sort='', scope_uris=''):
        """
        Returns a list of logical enclosures matching the specified filter. A maximum of 40 logical enclosures are
        returned to the caller. Additional calls can be made to retrieve any other logical enclosures matching the
        filter. Valid filter parameters include attributes of a Logical Enclosure resource.

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items.
                The actual number of items in the response might differ from the requested
                count if the sum of start and count exceeds the total number of items.
            filter (list or str):
                A general filter/query string to narrow the list of items returned. The
                default is no filter; all resources are returned.
            sort:
                The sort order of the returned data set. By default, the sort order is based
                on create time with the oldest entry first.
            scope_uris:
                An expression to restrict the resources returned according to the scopes to
                which they are assigned.

        Returns:
            list: A list of logical enclosures.
        """
        return self._helper.get_all(start, count, filter=filter,
                                    sort=sort, scope_uris=scope_uris)

    @ensure_resource_client
    def update_configuration(self, timeout=-1):
        """
        Reapplies the appliance's configuration on enclosures for the logical enclosure by ID or URI. This includes
        running the same configure steps that were performed as part of the enclosure add.

        Args:
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: Logical enclosure.
        """
        uri = "{}/configuration".format(self.data["uri"])
        updated_configuration = self._helper.update(None, uri, timeout=timeout)
        self.refresh()

        return updated_configuration

    @ensure_resource_client
    def get_script(self):
        """
        Gets the configuration script of the logical enclosure by ID or URI.

        Return:
            str: Configuration script.
        """
        uri = "{}/script".format(self.data["uri"])
        return self._helper.do_get(uri)

    @ensure_resource_client
    def update_script(self, information, timeout=-1):
        """
        Updates the configuration script of the logical enclosure and on all enclosures in the logical enclosure with
        the specified ID.

        Args:
            information: Updated script.
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Return:
            Configuration script.
        """
        uri = "{}/script".format(self.data["uri"])
        return self._helper.update(information, uri=uri, timeout=timeout)

    @ensure_resource_client
    def generate_support_dump(self, information, timeout=-1):
        """
        Generates a support dump for the logical enclosure with the specified ID. A logical enclosure support dump
        includes content for logical interconnects associated with that logical enclosure. By default, it also contains
        appliance support dump content.

        Args:
            information (dict): Information to generate support dump.
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: Support dump.
        """
        uri = "{}/support-dumps".format(self.data["uri"])
        return self._helper.create(information, uri=uri, timeout=timeout)

    @ensure_resource_client
    def update_from_group(self, data=None, timeout=-1):
        """
        Use this action to make a logical enclosure consistent with the enclosure group when the logical enclosure is
        in the Inconsistent state.

        Args:
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: Logical enclosure.
        """
        uri = "{}/updateFromGroup".format(self.data["uri"])
        return self._helper.update(data, uri, timeout=timeout)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import re

from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import (Resource, ResourceUtilizationMixin,
                                           ResourcePatchMixin, ensure_resource_client)

SERVER_HARWARE_BELOW_GEN10 = 'Server Hardware generation is below Gen10'
SERVER_PROFILE_ATTACHED = 'Server Hardware has a Profile attached'
SERVER_POWERED_ON = 'Server Hardware is in Powered On state'
ONGOING_FIRMWARE_UPDATE = 'Server Hardware is undergoing a firmware update'
ILO_ADVANCED_LICENSE_REQUIRED = 'Requires an HPE iLO Advanced license for monitored hardware'


class ServerHardware(ResourcePatchMixin, ResourceUtilizationMixin, Resource):
    """
    The server hardware resource is a representation of a physical server.
    The server hardware resource provides methods for server management tasks such
    as applying a profile, importing a server and managing an iLO.

    """
    URI = '/rest/server-hardware'

    def __init__(self, connection, data=None):
        super(ServerHardware, self).__init__(connection, data)

    def add(self, information, timeout=-1):
        """
        Adds a rack-mount server for management by the appliance. This API initiates the asynchronous addition of
        supported server models.

        Note: Servers in an enclosure are added by adding the enclosure resource. This is
        only supported on appliances that support rack-mounted servers.

        Args:
            information (dict): Object to create
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: Created rack-mount server.
        """
        return self.create(information, timeout=timeout)

    def add_multiple_servers(self, information, timeout=-1):
        """
        Adds multiple rack-mount servers for management by the appliance. This API initiates the asynchronous addition of
        supported server models.

        Note: Servers in an enclosure are added by adding the enclosure resource. This is
        only supported on appliances that support rack-mounted servers.

        This is only supported for api version 600

        Args:
            information (dict): Objects to create
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: Created rack-mount servers.
        """
        uri = "{}/discovery".format(self.URI)
        return self.create(information, uri=uri, timeout=timeout)

    def remove(self, force=False, timeout=-1):
        """
        Removes the rackserver with the specified URI.
        Note: This operation is only supported on appliances that support rack-mounted servers.

        Args:
            force (bool):
                If set to true, the operation completes despite any problems with
                network connectivity or errors on the resource itself. The default is false.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            bool: Indicates whether the resource was successfully removed.
        """
        return self.delete(force=force, timeout=timeout)

    @ensure_resource_client
    def get_bios(self):
        """
        Gets the list of BIOS/UEFI values currently set on the physical server.

        Returns:
            dict: Dictionary of BIOS/UEFI values.
        """
        uri = "{}/bios".format(self.data["uri"])
        return self._helper.do_get(uri)

    @ensure_resource_client
    def get_environmental_configuration(self):
        """
        Gets the settings that describe the environmental configuration (supported feature set, calibrated minimum and
        maximum power, location and dimensions, etc.) of the server hardware resource.

        Returns:
            dict: Environmental configuration settings.
        """
        uri = "{}/environmentalConfiguration".format(self.data["uri"])
        return self._helper.do_get(uri)

    @ensure_resource_client
    def update_environmental_configuration(self, configuration, timeout=-1):
        """
        Sets the calibrated max power of an unmanaged or unsupported server hardware resource.

        Args:
            configuration (dict): Environmental configuration.
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: Environmental configuration settings.
        """
        uri = "{}/environmentalConfiguration".format(self.data["uri"])
        return self._helper.update(configuration, uri, timeout=timeout)

    @ensure_resource_client
    def get_ilo_sso_url(self, ip=None):
        """
        Retrieves the URL to launch a Single Sign-On (SSO) session for the iLO web interface. If the server hardware is
        unsupported, the resulting URL will not use SSO and the iLO web interface will prompt for credentials.
        This is not supported on G7/iLO3 or earlier servers.

        Args:
            ip: IP address or host name of the server's iLO management processor

        Returns:
            URL
        """
        uri = "{}/iloSsoUrl".format(self.data["uri"])

        if ip:
            uri = "{}?ip={}".format(uri, ip)

        return self._helper.do_get(uri)

    def get_all_firmwares(self, filter='', start=0, count=-1, query='', sort=''):
        """
        Gets a list of firmware inventory across all servers. To filter the returned data, specify a filter
        expression to select a particular server model, component name, and/or component firmware version.

        Note:
            This method is available for API version 300 or later.

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items.
                The actual number of items in the response might differ from the requested
                count if the sum of start and count exceeds the total number of items.
            filter (list or str):
                A general filter/query string to narrow the list of items returned. The
                default is no filter; all resources are returned.
            query:
                A general query string to narrow the list of resources returned. The default is no query; all resources
                are returned.
            sort:
                The sort order of the returned data set. By default, the sort order is based
                on create time with the oldest entry first.

        Returns:
            list: List of firmware inventory.
        """
        uri = self.URI + "/*/firmware"
        return self._helper.get_all(start, count, filter, query, sort, '', '', uri)

    @ensure_resource_client
    def get_firmware(self):
        """
        Get the firmware inventory of a server.

        Note:
            This method is available for API version 300 or later.

        Returns:
            dict: Server Hardware firmware.
        """
        uri = "{}/firmware".format(self.data["uri"])
        return self._helper.do_get(uri)

    @ensure_resource_client
    def get_java_remote_console_url(self, ip=None):
        """
        Generates a Single Sign-On (SSO) session for the iLO Java Applet console and returns the URL to launch it.
        If the server hardware is unmanaged or unsupported, the resulting URL will not use SSO and the iLO Java Applet
        will prompt for credentials. This is not supported on G7/iLO3 or earlier servers.

        Args:
            ip: IP address or host name of the server's iLO management processor

        Returns:
            URL
        """
        uri = "{}/javaRemoteConsoleUrl".format(self.data["uri"])

        if ip:
            uri = "{}?ip={}".format(uri, ip)

        return self._helper.do_get(uri)

    @ensure_resource_client
    def update_mp_firware_version(self, timeout=-1):
        """
        Updates the iLO firmware on a physical server to a minimum ILO firmware version required by OneView to
        manage the server.

        Args:
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
        Returns:
            Resource
        """
        uri = "{}/mpFirmwareVersion".format(self.data["uri"])
        return self._helper.do_put(uri, None, timeout, None)

    @ensure_resource_client
    def update_power_state(self, configuration, timeout=-1):
        """
        Refreshes the server hardware to fix configuration issues.

        Args:
            configuration (dict): Power state configuration.
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            Resource
        """
        uri = "{}/powerState".format(self.data["uri"])
        return self._helper.update(configuration, uri, timeout=timeout)

    @ensure_resource_client
    def refresh_state(self, configuration, timeout=-1):
        """
        Refreshes the server hardware to fix configuration issues.

        Args:
            configuration: Refresh state configuration.
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            Resource
        """
        uri = "{}/refreshState".format(self.data["uri"])
        return self._helper.update(configuration, uri=uri, timeout=timeout)

    @ensure_resource_client
    def get_remote_console_url(self, ip=None, consoleType=None):
        """
        Generates a Single Sign-On (SSO) session for the iLO Integrated Remote Console Application (IRC) and returns the
        URL to launch it. If the server hardware is unmanaged or unsupported, the resulting URL will not use SSO and the
        IRC application will prompt for credentials. Use of this URL requires a previous installation of the iLO IRC and
        is supported only on Windows clients.

        Args:
            ip: IP address or host name of the server's iLO management processor
            consoleType: Type of console requested

        Returns:
            URL
        """
        uri = "{}/remoteConsoleUrl".format(self.data["uri"])

        if ip:
            uri = "{}?ip={}".format(uri, ip)

        elif consoleType:
            uri = "{}?consoleType={}".format(uri, consoleType)

        return self._helper.do_get(uri)

    @ensure_resource_client
    def get_physical_server_hardware(self):
        """
        Information describing an 'SDX' partition including a list of physical server blades represented by a server
        hardware. Used with SDX enclosures only.

        Returns:
            Resource
        """
        uri = "{}/physicalServerHardware".format(self.data["uri"])
        return self._helper.do_get(uri)

    @ensure_resource_client
    def get_local_storage(self, ip=None):
        """
        Gets the updated version 2 local storage resource for the server, including storage controllers,
        drives, and volumes.

        Args:
            ip: IP address or host name of the server's iLO management processor

        Returns:
            Resource
        """
        uri = "{}/localStorageV2".format(self.data["uri"])

        if ip:
            uri = "{}?ip={}".format(uri, ip)

        return self._helper.do_get(uri)

    def check_firmware_compliance(self, configuration, timeout=-1, custom_headers=None):
        """
        Checks the firmware compliance of a server with the selected firmware baseline.
        Returns the compliance status of each individual component as well as the overall
        firmware compliance status of the server.

        Args:
            configuration: Firmware Compliance Configuration

        Returns:
            Resource
        """
        uri = "{}/firmware-compliance".format(self.URI)
        return self._helper.do_post(uri, configuration, timeout=timeout, custom_headers=custom_headers)

    @ensure_resource_client
    def validate_server_hardware_for_firmware_update(self):
        """
        Performs a validation for the prerequisites before firmware update.
        Checks if the server hardware is in powered off state, No server profile attached to server
        hardware, server hardware model is Gen10 or above, server hardware has advanced ilo license type.

        """
        state = self.data["state"]
        server_profile_uri = self.data["serverProfileUri"]
        power_state = self.data["powerState"]
        generation = self.data["mpModel"]
        license_type = self.data["mpLicenseType"]

        validation_error_list = []

        matches = re.findall("\d+$", generation)
        if int(matches[0]) < 5:
            validation_error_list.append(SERVER_HARWARE_BELOW_GEN10)
        if server_profile_uri:
            validation_error_list.append(SERVER_PROFILE_ATTACHED)
        if state == "UpdatingFirmware":
            validation_error_list.append(ONGOING_FIRMWARE_UPDATE)
        if power_state != "Off":
            validation_error_list.append(SERVER_POWERED_ON)
        if license_type != "iLO Advanced":
            validation_error_list.append(ILO_ADVANCED_LICENSE_REQUIRED)

        if len(validation_error_list) > 0:
            raise ValueError(validation_error_list)
        else:
            return True

    @ensure_resource_client
    def perform_firmware_update(self, configuration, timeout=-1, custom_headers=None):
        """
        Performs a specific patch operation on the firmware settings for the given server.
        If the server supports the particular operation, the operation is performed and a response is
        returned to the caller with the results.

        Args:
            configuration: Firmware Update Configuration

        Returns:
            Updated Resource
        """
        uri = "{}/firmware/settings".format(self.data["uri"])
        if self.validate_server_hardware_for_firmware_update():
            return self.patch_request(uri, configuration, timeout=timeout, custom_headers=custom_headers)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2021] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library
from future.utils import lmap

standard_library.install_aliases()

import logging
import os
from copy import deepcopy
from urllib.parse import quote
from functools import partial

from hpeOneView.resources.task_monitor import TaskMonitor
from hpeOneView import exceptions

RESOURCE_CLIENT_RESOURCE_WAS_NOT_PROVIDED = 'Resource was not provided'
RESOURCE_CLIENT_INVALID_FIELD = 'Invalid field was provided'
RESOURCE_CLIENT_INVALID_ID = 'Invalid id was provided'
RESOURCE_CLIENT_UNKNOWN_OBJECT_TYPE = 'Unknown object type'
UNRECOGNIZED_URI = 'Unrecognized URI for this resource'
RESOURCE_CLIENT_TASK_EXPECTED = "Failed: Expected a TaskResponse."
RESOURCE_ID_OR_URI_REQUIRED = 'It is required to inform the Resource ID or URI.'
UNAVAILABLE_METHOD = "Method is not available for this resource"
MISSING_UNIQUE_IDENTIFIERS = "Missing unique identifiers(URI/Name) for the resource"
RESOURCE_DOES_NOT_EXIST = "Resource does not exist with the provided unique identifiers"

logger = logging.getLogger(__name__)


class EnsureResourceClient(object):
    """Decorator class to update the resource data."""

    def __init__(self, method=None, update_data=False):
        self.method = method
        self.update_data = update_data

    def __get__(self, obj, objtype):
        return partial(self.__call__, obj)

    def __call__(self, obj, *args, **kwargs):
        if self.method:
            obj.ensure_resource_data(update_data=self.update_data)
            return self.method(obj, *args, **kwargs)

        def wrap(*args, **kwargs):
            args[0].ensure_resource_data(update_data=self.update_data)
            return obj(*args, **kwargs)

        return wrap


# Decorator to ensure the resource client
ensure_resource_client = EnsureResourceClient


class Resource(object):
    """Base class for OneView resources.

    Args:
        connection: OneView connection object
        data: Resource data
    """
    # Base URI for the rest calls
    URI = '/rest'

    # Unique identifiers to query the resource
    UNIQUE_IDENTIFIERS = ['uri', 'name']

    # Default values required for the api versions
    DEFAULT_VALUES = {}

    def __init__(self, connection, data=None):
        self._connection = connection
        self._task_monitor = TaskMonitor(connection)
        self._helper = ResourceHelper(self.URI,
                                      self._connection,
                                      self._task_monitor)
        # Resource data
        self.data = data if data else {}

        # Merge resoure data with the default values
        self._merge_default_values()

    def ensure_resource_data(self, update_data=False):
        """Retrieves data from OneView and updates resource object.

        Args:
            update_data: Flag to update resource data when it is required.
        """
        # Check for unique identifier in the resource data
        if not any(key in self.data for key in self.UNIQUE_IDENTIFIERS):
            raise exceptions.HPEOneViewMissingUniqueIdentifiers(MISSING_UNIQUE_IDENTIFIERS)

        # Returns if data update is not required
        if not update_data:
            return

        resource_data = None

        if 'uri' in self.UNIQUE_IDENTIFIERS and self.data.get('uri'):
            resource_data = self._helper.do_get(self.data['uri'])
        else:
            for identifier in self.UNIQUE_IDENTIFIERS:
                identifier_value = self.data.get(identifier)

                if identifier_value:
                    result = self.get_by(identifier, identifier_value)
                    if result and isinstance(result, list):
                        resource_data = result[0]
                        break

        if resource_data:
            self.data.update(resource_data)
        else:
            raise exceptions.HPEOneViewResourceNotFound(RESOURCE_DOES_NOT_EXIST)

    @ensure_resource_client
    def refresh(self):
        """Helps to get the latest resource data from the server."""
        self.data = self._helper.do_get(self.data["uri"])

    def get_all(self, start=0, count=-1, filter='', sort=''):
        """Gets all items according with the given arguments.

        Args:
            start: The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count: The number of resources to return. A count of -1 requests all items (default).
            filter (list or str): A general filter/query string to narrow the list of items returned. The default is no
                filter; all resources are returned.
            sort: The sort order of the returned data set. By default, the sort order is based on create time with the
                oldest entry first.

        Returns:
            list: A list of items matching the specified filter.
        """
        result = self._helper.get_all(start=start, count=count, filter=filter, sort=sort)
        return result

    def create(self, data=None, uri=None, timeout=-1, custom_headers=None, force=False):
        """Makes a POST request to create a resource when a request body is required.

        Args:
            data: Additional fields can be passed to create the resource.
            uri: Resouce uri
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers: Allows set specific HTTP headers.
        Returns:
            Created resource.
        """
        if not data:
            data = {}

        default_values = self._get_default_values()
        data = self._helper.update_resource_fields(data, default_values)

        logger.debug('Create (uri = %s, resource = %s)' % (uri, str(data)))
        resource_data = self._helper.create(data, uri, timeout, custom_headers, force)
        new_resource = self.new(self._connection, resource_data)

        return new_resource

    @ensure_resource_client
    def delete(self, timeout=-1, custom_headers=None, force=False):
        """Deletes current resource.

        Args:
            timeout: Timeout in seconds.
            custom_headers: Allows to set custom http headers.
            force: Flag to force the operation.
        """
        uri = self.data['uri']

        logger.debug("Delete resource (uri = %s)" % (str(uri)))

        return self._helper.delete(uri, timeout=timeout,
                                   custom_headers=custom_headers, force=force)

    @ensure_resource_client(update_data=True)
    def update(self, data=None, timeout=-1, custom_headers=None, force=False):
        """Makes a PUT request to update a resource when a request body is required.

        Args:
            data: Data to update the resource.
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers: Allows to add custom HTTP headers.
            force: Force the update operation.

        Returns:
            A dict with the updated resource data.
        """
        uri = self.data['uri']

        resource = deepcopy(self.data)
        resource.update(data)

        logger.debug('Update async (uri = %s, resource = %s)' %
                     (uri, str(resource)))

        self.data = self._helper.update(resource, uri, force, timeout, custom_headers)

        return self

    def get_by(self, field, value):
        """Get the resource by passing a field and its value.

        Note:
            This function uses get_all passing a filter.The search is case-insensitive.

        Args:
            field: Field name to filter.
            value: Value to filter.

        Returns:
            dict
        """
        if not field:
            logger.exception(RESOURCE_CLIENT_INVALID_FIELD)
            raise ValueError(RESOURCE_CLIENT_INVALID_FIELD)

        filter = "\"{0}='{1}'\"".format(field, value)
        results = self.get_all(filter=filter)

        # Workaround when the OneView filter does not work, it will filter again
        if "." not in field:
            # This filter only work for the first level
            results = [item for item in results if str(item.get(field, "")).lower() == value.lower()]
        return results

    def get_by_name(self, name):
        """Retrieves a resource by its name.

        Args:
            name: Resource name.

        Returns:
            Resource object or None if resource does not exist.
        """
        result = self.get_by("name", name)
        if result:
            data = result[0]
            new_resource = self.new(self._connection, data)
        else:
            new_resource = None
        return new_resource

    # Sometimes get_all() with filters are not returning correct values, so added this method to overcome that issue
    def get_by_field(self, field, value):
        """Retrieves a resource by its field.

        Args:
            field: Resource field name.
            value: Resource field value.

        Returns:
            Resource object or None if resource does not exist.
        """
        if not field:
            logger.exception(RESOURCE_CLIENT_INVALID_FIELD)
            raise ValueError(RESOURCE_CLIENT_INVALID_FIELD)

        results = self.get_all()

        # This filter only work for the first level
        result = [item for item in results if value.lower() in str(
            item.get(field, "")).lower()]

        if result:
            data = result[0]
            new_resource = self.new(self._connection, data)
        else:
            new_resource = None
        return new_resource

    def get_by_uri(self, uri):
        """Retrieves a resource by its URI

        Args:
            uri: URI of the resource

        Returns:
            Resource object
        """
        self._helper.validate_resource_uri(uri)
        data = self._helper.do_get(uri)

        if data:
            new_resource = self.new(self._connection, data)
        else:
            new_resource = None

        return new_resource

    def get_by_id(self, id):
        """Retrieves a resource by its id.

        Args:
            id: id of Resource

        Returns:
            Resource object or None if resource does not exist.

        """

        uri = "{}/{}".format(self.URI, id)
        self._helper.validate_resource_uri(uri)
        data = self._helper.do_get(uri)

        if data:
            new_resource = self.new(self._connection, data)
        else:
            new_resource = None

        return new_resource

    def _get_default_values(self, default_values=None):
        """Gets the default values set for a resource"""

        if not default_values:
            default_values = self.DEFAULT_VALUES

        if default_values:
            api_version = str(self._connection._apiVersion)
            values = default_values.get(api_version, {}).copy()
        else:
            values = {}

        return values

    def _merge_default_values(self):
        """Merge default values with resource data."""
        values = self._get_default_values()
        for key, value in values.items():
            if not self.data.get(key):
                self.data[key] = value

    @classmethod
    def new(cls, connection, data):
        """Returns a new resource object"""
        return cls(connection, data)


class ResourceHelper(object):

    def __init__(self, base_uri, connection, task_monitor):
        self._base_uri = base_uri
        self._connection = connection
        self._task_monitor = task_monitor

    def get_all(self, start=0, count=-1, filter='', query='', sort='', view='', fields='', uri=None, scope_uris='', custom_headers=None,
                name_prefix='', category=[], childLimit=0, topCount=0,):
        """Gets all items according with the given arguments.

        Args:
            start: The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count: The number of resources to return. A count of -1 requests all items (default).
            filter (list or str): A general filter/query string to narrow the list of items returned. The default is no
                filter; all resources are returned.
            query: A single query parameter can do what would take multiple parameters or multiple GET requests using
                filter. Use query for more complex queries. NOTE: This parameter is experimental for OneView 2.0.
            sort: The sort order of the returned data set. By default, the sort order is based on create time with the
                oldest entry first.
            view:
                Returns a specific subset of the attributes of the resource or collection by specifying the name of a
                predefined view. The default view is expand (show all attributes of the resource and all elements of
                the collections or resources).
            fields:
                Name of the fields.
            uri:
                A specific URI (optional)
            scope_uris:
                An expression to restrict the resources returned according to the scopes to
                which they are assigned.
            custom_headers: custom headers

        Returns:
             list: A list of items matching the specified filter.
        """
        if not uri:
            uri = self._base_uri

        uri = self.build_query_uri(uri=uri,
                                   start=start,
                                   count=count,
                                   filter=filter,
                                   query=query,
                                   sort=sort,
                                   view=view,
                                   fields=fields,
                                   scope_uris=scope_uris,
                                   name_prefix=name_prefix,
                                   category=category,
                                   childLimit=childLimit,
                                   topCount=topCount)

        logger.debug('Getting all resources with uri: {0}'.format(uri))

        return self.do_requests_to_getall(uri, count, custom_headers=custom_headers)

    def delete_all(self, filter, force=False, timeout=-1):
        """
        Deletes all resources from the appliance that match the provided filter.

        Args:
            filter:
                A general filter/query string to narrow the list of items deleted.
            force:
                If set to true, the operation completes despite any problems with network connectivity or errors
                on the resource itself. The default is false.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            bool: Indicates if the resources were successfully deleted.
        """
        uri = "{}?filter={}&force={}".format(self._base_uri, quote(filter), force)
        logger.debug("Delete all resources (uri = %s)" % uri)

        return self.delete(uri)

    def create(self, data=None, uri=None, timeout=-1, custom_headers=None, force=False):
        """Makes a POST request to create a resource when a request body is required.

        Args:
            data: Additional fields can be passed to create the resource.
            uri: Resouce uri
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers: Allows set specific HTTP headers.
        Returns:
            Created resource.
        """
        if not uri:
            uri = self._base_uri

        if force:
            uri += '?force={}'.format(force)

        logger.debug('Create (uri = %s, resource = %s)' % (uri, str(data)))
        return self.do_post(uri, data, timeout, custom_headers)

    def delete(self, uri, force=False, timeout=-1, custom_headers=None):
        """Deletes current resource.

        Args:
            force: Flag to delete the resource forcefully, default is False.
            timeout: Timeout in seconds.
            custom_headers: Allows to set custom http headers.
        """
        if force:
            uri += '?force=True'

        logger.debug("Delete resource (uri = %s)" % (str(uri)))

        task, body = self._connection.delete(uri, custom_headers=custom_headers)

        if not task:
            # 204 NO CONTENT
            # Successful return from a synchronous delete operation.
            return True

        task = self._task_monitor.wait_for_task(task, timeout=timeout)

        return task

    def update(self, resource, uri=None, force=False, timeout=-1, custom_headers=None):
        """Makes a PUT request to update a resource when a request body is required.

        Args:
            resource: Data to update the resource.
            uri: Resource uri
            force: If set to true, the operation completes despite any problems
                with network connectivity or errors on the resource itself. The default is false.
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers: Allows to add custom HTTP headers.

        Returns:
            A dict with the updated resource data.
        """
        logger.debug('Update async (uri = %s, resource = %s)' %
                     (uri, str(resource)))
        if not uri:
            uri = resource['uri']

        if force:
            uri += '?force=True'

        return self.do_put(uri, resource, timeout, custom_headers)

    def update_with_zero_body(self, uri, timeout=-1, custom_headers=None):
        """Makes a PUT request to update a resource when no request body is required.

        Args:
            uri: Allows to use a different URI other than resource URI
            timeout: Timeout in seconds. Wait for task completion by default.
                The timeout does not abort the operation in OneView; it just stops waiting for its completion.
            custom_headers: Allows to set custom HTTP headers.

        Returns:
            A dict with updated resource data.
        """
        logger.debug('Update with zero length body (uri = %s)' % uri)

        return self.do_put(uri, None, timeout, custom_headers)

    def create_report(self, uri, timeout=-1):
        """
        Creates a report and returns the output.

        Args:
            uri: URI
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            list:
        """
        logger.debug('Creating Report')
        task, _ = self._connection.post(uri, {})

        if not task:
            raise exceptions.HPEOneViewException(RESOURCE_CLIENT_TASK_EXPECTED)

        task = self._task_monitor.get_completed_task(task, timeout)

        return task['taskOutput']

    def get_collection(self, uri=None, filter='', path=''):
        """Retrieves a collection of resources.

        Use this function when the 'start' and 'count' parameters are not allowed in the GET call.
        Otherwise, use get_all instead.

        Optional filtering criteria may be specified.

        Args:
            filter (list or str): General filter/query string.
            path (str): path to be added with base URI

        Returns:
             Collection of the requested resource.
        """
        if not uri:
            uri = self._base_uri

        if filter:
            filter = self.make_query_filter(filter)
            filter = "?" + filter[1:]

        uri = "{uri}{path}{filter}".format(uri=uri, path=path, filter=filter)
        logger.debug('Get resource collection (uri = %s)' % uri)

        response = self._connection.get(uri)

        return self.get_members(response)

    def build_query_uri(self, uri=None, start=0, count=-1, filter='', query='', sort='', view='', fields='', scope_uris='',
                        name_prefix='', category=[], childLimit=0, topCount=0):
        """Builds the URI from given parameters.

        More than one request can be send to get the items, regardless the query parameter 'count', because the actual
        number of items in the response might differ from the requested count. Some types of resource have a limited
        number of items returned on each call. For those resources, additional calls are made to the API to retrieve
        any other items matching the given filter. The actual number of items can also differ from the requested call
        if the requested number of items would take too long.

        The use of optional parameters for OneView 2.0 is described at:
        http://h17007.www1.hpe.com/docs/enterprise/servers/oneview2.0/cic-api/en/api-docs/current/index.html

        Note:
            Single quote - "'" - inside a query parameter is not supported by OneView API.

        Args:
            start: The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count: The number of resources to return. A count of -1 requests all items (default).
            filter (list or str): A general filter/query string to narrow the list of items returned. The default is no
                filter; all resources are returned.
            query: A single query parameter can do what would take multiple parameters or multiple GET requests using
                filter. Use query for more complex queries. NOTE: This parameter is experimental for OneView 2.0.
            sort: The sort order of the returned data set. By default, the sort order is based on create time with the
                oldest entry first.
            view: Returns a specific subset of the attributes of the resource or collection by specifying the name of a
                predefined view. The default view is expand (show all attributes of the resource and all elements of
                the collections or resources).
            fields: Name of the fields.
            uri: A specific URI (optional)
            scope_uris: An expression to restrict the resources returned according to the scopes to
                which they are assigned.
            name_prefix: Filters the resource returned by the given prefix.

        Returns:
            uri: The complete uri
        """
        if filter:
            filter = self.make_query_filter(filter)

        if query:
            query = "&query=" + quote(query)

        if sort:
            sort = "&sort=" + quote(sort)

        if view:
            view = "&view=" + quote(view)

        if fields:
            fields = "&fields=" + quote(fields)

        if scope_uris:
            scope_uris = "&scopeUris=" + quote(scope_uris)

        if name_prefix:
            name_prefix = "&namePrefix=" + quote(name_prefix)

        categories = ''
        if category:
            for cat in category:
                categories = categories + "&category=" + quote(cat)

        if childLimit:
            childLimit = "&childLimit=" + str(childLimit)

        if topCount:
            topCount = "&topCount=" + str(topCount)

        path = uri if uri else self._base_uri

        self.validate_resource_uri(path)

        symbol = '?' if '?' not in path else '&'

        uri = "{0}{1}start={2}&count={3}{4}{5}{6}{7}{8}{9}{10}{11}".format(path, symbol, start, count, filter, query, sort,
                                                                           view, fields, scope_uris, name_prefix, categories)
        return uri

    def build_uri_with_query_string(self, kwargs, sufix_path='', uri=None):
        if not uri:
            uri = self._base_uri

        query_string = '&'.join('{}={}'.format(key, kwargs[key]) for key in sorted(kwargs))
        return uri + sufix_path + '?' + query_string

    def build_uri(self, id_or_uri):
        """Helps to build the URI from resource id and validate the URI.

        Args:
            id_or_uri: ID/URI of the resource.

        Returns:
            Returns a valid resource URI
        """
        if not id_or_uri:
            logger.exception(RESOURCE_CLIENT_INVALID_ID)
            raise ValueError(RESOURCE_CLIENT_INVALID_ID)

        if "/" in id_or_uri:
            self.validate_resource_uri(id_or_uri)
            return id_or_uri
        else:
            return self._base_uri + "/" + id_or_uri

    def build_subresource_uri(self, resource_id_or_uri=None, subresource_id_or_uri=None, subresource_path=''):
        """Helps to build a URI with resource path and its sub resource path.

        Args:
            resoure_id_or_uri: ID/URI of the main resource.
            subresource_id__or_uri: ID/URI of the sub resource.
            subresource_path: Sub resource path to be added with the URI.

        Returns:
            Returns URI
        """
        if subresource_id_or_uri and "/" in subresource_id_or_uri:
            return subresource_id_or_uri
        else:
            if not resource_id_or_uri:
                raise exceptions.HPEOneViewValueError(RESOURCE_ID_OR_URI_REQUIRED)

            resource_uri = self.build_uri(resource_id_or_uri)

            uri = "{}/{}/{}".format(resource_uri, subresource_path, str(subresource_id_or_uri or ''))
            uri = uri.replace("//", "/")

            if uri.endswith("/"):
                uri = uri[:-1]

            return uri

    def validate_resource_uri(self, path):
        """Helper method to validate URI of the resource."""
        if self._base_uri not in path:
            logger.exception('Get by uri : unrecognized uri: (%s)' % path)
            raise exceptions.HPEOneViewUnknownType(UNRECOGNIZED_URI)

    def make_query_filter(self, filters):
        """Helper method to build filter query parameter."""
        if isinstance(filters, list):
            formated_filter = "&filter=".join(quote(f) for f in filters)
        else:
            formated_filter = quote(filters)

        return "&filter=" + formated_filter

    def get_members(self, mlist):
        """Get members from list of resources"""
        if mlist and mlist.get('members'):
            return mlist['members']
        else:
            return []

    def update_resource_fields(self, data, data_to_add):
        """Update resource data with new fields.

        Args:
            data: resource data
            data_to_update: dict of data to update resource data

        Returnes:
            Returnes dict
        """
        for key, value in data_to_add.items():
            if not data.get(key):
                data[key] = value

        return data

    def do_requests_to_getall(self, uri, requested_count, custom_headers=None):
        """Helps to make http request for get_all method.

        Note:
            This method will be checking for the pagination URI in the response
            and make request to pagination URI to get all the resources.
        """
        items = []
        while uri:
            logger.debug('Making HTTP request to get all resources. Uri: {0}'.format(uri))
            response = self._connection.get(uri, custom_headers=custom_headers)
            members = self.get_members(response)
            items += members

            logger.debug("Response getAll: nextPageUri = {0}, members list length: {1}".format(uri, str(len(members))))
            uri = self.get_next_page(response, items, requested_count)

        logger.debug('Total # of members found = {0}'.format(str(len(items))))
        return items

    def get_next_page(self, response, items, requested_count):
        """Returns next page URI."""
        next_page_is_empty = response.get('nextPageUri') is None
        has_different_next_page = not response.get('uri') == response.get('nextPageUri')
        has_next_page = not next_page_is_empty and has_different_next_page

        if len(items) >= requested_count and requested_count != -1:
            return None

        return response.get('nextPageUri') if has_next_page else None

    def do_get(self, uri):
        """Helps to make get requests

        Args:
            uri: URI of the resource

        Returns:
            Returns: Returns the resource data
        """
        self.validate_resource_uri(uri)
        return self._connection.get(uri)

    def do_post(self, uri, resource, timeout, custom_headers):
        """Helps to make post requests.

        Args:
            uri: URI of  the resource.
            resource: Resource data to post.
            timeout: Time out for the request in seconds.
            cutom_headers: Allows to add custom http headers.

        Returns:
            Retunrs Task object.
        """
        self.validate_resource_uri(uri)
        task, entity = self._connection.post(uri, resource, custom_headers=custom_headers)

        if not task:
            return entity

        return self._task_monitor.wait_for_task(task, timeout)

    def do_put(self, uri, resource, timeout, custom_headers):
        """Helps to make put requests.

        Args:
            uri: URI of the resource
            timeout: Time out for the request in seconds.
            custom_headers: Allows to set custom http headers.

        Retuns:
            Returns Task object
        """
        self.validate_resource_uri(uri)

        task, body = self._connection.put(uri, resource, custom_headers=custom_headers)

        if not task:
            return body

        return self._task_monitor.wait_for_task(task, timeout)

    def add_new_fields(data, data_to_add):
        """Update resource data with new fields.

        Args:
            data: resource data
            data_to_update: dict of data to update resource data

        Returnes:
            Returnes dict
        """
        for key, value in data_to_add.items():
            if not data.get(key):
                data[key] = value

        return data


class ResourcePatchMixin(object):

    @ensure_resource_client
    def patch(self, operation, path, value, custom_headers=None, timeout=-1):
        """Uses the PATCH to update a resource.

        Only one operation can be performed in each PATCH call.

        Args
            operation: Patch operation
            path: Path
            value: Value
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers: Allows to add custom http headers.

        Returns:
            Updated resource.
        """
        patch_request_body = [{'op': operation, 'path': path, 'value': value}]
        resource_uri = self.data['uri']

        self.data = self.patch_request(resource_uri,
                                       body=patch_request_body,
                                       custom_headers=custom_headers,
                                       timeout=timeout)
        return self

    def patch_request(self, uri, body, custom_headers=None, timeout=-1):
        """Uses the PATCH to update a resource.

        Only one operation can be performed in each PATCH call.

        Args:
            body (list): Patch request body
            timeout (int): Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers (dict): Allows to add custom http headers.

        Returns:
            Updated resource.
        """
        logger.debug('Patch resource (uri = %s, data = %s)' % (uri, body))

        if not custom_headers:
            custom_headers = {}

        if self._connection._apiVersion >= 300 and 'Content-Type' not in custom_headers:
            custom_headers['Content-Type'] = 'application/json-patch+json'

        task, entity = self._connection.patch(uri, body, custom_headers=custom_headers)

        if not task:
            return entity

        return self._task_monitor.wait_for_task(task, timeout)


class ResourceFileHandlerMixin(object):

    def upload(self, file_path, uri=None, timeout=-1):
        """Makes a multipart request.

        Args:
            file_path: File to upload.
            uri: A specific URI (optional).
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: Response body.
        """
        if not uri:
            uri = self.URI

        upload_file_name = os.path.basename(file_path)
        task, entity = self._connection.post_multipart_with_response_handling(uri, file_path, upload_file_name)

        if not task:
            return entity

        return self._task_monitor.wait_for_task(task, timeout)

    def download(self, uri, file_path):
        """Downloads the contents of the requested URI to a stream.

        Args:
            uri: URI
            file_path: File path destination

        Returns:
            bool: Indicates if the file was successfully downloaded.
        """
        with open(file_path, 'wb') as file:
            return self._connection.download_to_stream(file, uri)


class ResourceUtilizationMixin(object):

    def get_utilization(self, fields=None, filter=None, refresh=False, view=None):
        """Retrieves historical utilization data for the specified resource, metrics, and time span.

        Args:
            fields: Name of the supported metric(s) to be retrieved in the format METRIC[,METRIC]...
                If unspecified, all metrics supported are returned.

            filter (list or str): Filters should be in the format FILTER_NAME=VALUE[,FILTER_NAME=VALUE]...
                E.g.: 'startDate=2016-05-30T11:20:44.541Z,endDate=2016-05-30T19:20:44.541Z'

                startDate
                    Start date of requested starting time range in ISO 8601 format. If omitted, the startDate is
                    determined by the endDate minus 24 hours.
                endDate
                    End date of requested starting time range in ISO 8601 format. When omitted, the endDate includes
                    the latest data sample available.

                If an excessive number of samples would otherwise be returned, the results will be segmented. The
                caller is responsible for comparing the returned sliceStartTime with the requested startTime in the
                response. If the sliceStartTime is greater than the oldestSampleTime and the requested start time,
                the caller is responsible for repeating the request with endTime set to sliceStartTime to obtain the
                next segment. This process is repeated until the full data set is retrieved.

                If the resource has no data, the UtilizationData is still returned but will contain no samples and
                sliceStartTime/sliceEndTime will be equal. oldestSampleTime/newestSampleTime will still be set
                appropriately (null if no data is available). If the filter does not happen to overlap the data
                that a resource has, then the metric history service will return null sample values for any
                missing samples.

            refresh: Specifies that if necessary, an additional request will be queued to obtain the most recent
                utilization data from the iLO. The response will not include any refreshed data. To track the
                availability of the newly collected data, monitor the TaskResource identified by the refreshTaskUri
                property in the response. If null, no refresh was queued.

            view: Specifies the resolution interval length of the samples to be retrieved. This is reflected in the
                resolution in the returned response. Utilization data is automatically purged to stay within storage
                space constraints. Supported views are listed below:

                native
                    Resolution of the samples returned will be one sample for each 5-minute time period. This is the
                    default view and matches the resolution of the data returned by the iLO. Samples at this resolution
                    are retained up to one year.
                hour
                    Resolution of the samples returned will be one sample for each 60-minute time period. Samples are
                    calculated by averaging the available 5-minute data samples that occurred within the hour, except
                    for PeakPower which is calculated by reporting the peak observed 5-minute sample value data during
                    the hour. Samples at this resolution are retained up to three years.
                day
                    Resolution of the samples returned will be one sample for each 24-hour time period. One day is a
                    24-hour period that starts at midnight GMT regardless of the time zone in which the appliance or
                    client is located. Samples are calculated by averaging the available 5-minute data samples that
                    occurred during the day, except for PeakPower which is calculated by reporting the peak observed
                    5-minute sample value data during the day. Samples at this resolution are retained up to three
                    years.

        Returns:
            dict
        """
        resource_uri = self.data['uri']
        query = ''

        if filter:
            query += self._helper.make_query_filter(filter)

        if fields:
            query += "&fields=" + quote(fields)

        if refresh:
            query += "&refresh=true"

        if view:
            query += "&view=" + quote(view)

        if query:
            query = "?" + query[1:]

        uri = "{0}/utilization{1}".format(self._helper.build_uri(resource_uri), query)

        return self._helper.do_get(uri)


class ResourceSchemaMixin(object):

    def get_schema(self):
        """Supports schema requests.

        Returns:
            A dict with the schema.
        """
        return self._helper.do_get(self.URI + '/schema')


class ResourceZeroBodyMixin(object):

    def create_with_zero_body(self, uri=None, timeout=-1, custom_headers=None):
        """Makes a POST request to create a resource when no request body is required.

        Args:
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers: Allows set specific HTTP headers.

        Returns:
            Created resource.
        """
        if not uri:
            uri = self.URI

        logger.debug('Create with zero body (uri = %s)' % uri)
        resource_data = self._helper.do_post(uri, {}, timeout, custom_headers)

        return resource_data

    def update_with_zero_body(self, uri=None, timeout=-1, custom_headers=None):
        """Makes a PUT request to update a resource when no request body is required.

        Args:
            uri: Allows to use a different URI other than resource URI
            timeout: Timeout in seconds. Wait for task completion by default.
                The timeout does not abort the operation in OneView; it just stops waiting for its completion.
            custom_headers: Allows to set custom HTTP headers.

        Returns:
            A dict with updated resource data.
        """
        if not uri:
            uri = self.data['uri']

        logger.debug('Update with zero length body (uri = %s)' % uri)
        resource_data = self._helper.do_put(uri, None, timeout, custom_headers)

        return resource_data


class ResourceClient(object):
    """
    This class implements common functions for HpOneView API rest
    """

    def __init__(self, con, uri):
        self._connection = con
        self._uri = uri
        self._task_monitor = TaskMonitor(con)

    def build_query_uri(self, start=0, count=-1, filter='', query='', sort='', view='', fields='', uri=None, scope_uris=''):
        """
        Builds the URI given the parameters.

        More than one request can be send to get the items, regardless the query parameter 'count', because the actual
        number of items in the response might differ from the requested count. Some types of resource have a limited
        number of items returned on each call. For those resources, additional calls are made to the API to retrieve
        any other items matching the given filter. The actual number of items can also differ from the requested call
        if the requested number of items would take too long.

        The use of optional parameters for OneView 2.0 is described at:
        http://h17007.www1.hpe.com/docs/enterprise/servers/oneview2.0/cic-api/en/api-docs/current/index.html

        Note:
            Single quote - "'" - inside a query parameter is not supported by OneView API.

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items (default).
            filter (list or str):
                A general filter/query string to narrow the list of items returned. The default is no
                filter; all resources are returned.
            query:
                A single query parameter can do what would take multiple parameters or multiple GET requests using
                filter. Use query for more complex queries. NOTE: This parameter is experimental for OneView 2.0.
            sort:
                The sort order of the returned data set. By default, the sort order is based on create time with the
                oldest entry first.
            view:
                Returns a specific subset of the attributes of the resource or collection by specifying the name of a
                predefined view. The default view is expand (show all attributes of the resource and all elements of
                the collections or resources).
            fields:
                Name of the fields.
            uri:
                A specific URI (optional)
            scope_uris:
                An expression to restrict the resources returned according to the scopes to
                which they are assigned.

        Returns:
            uri: The complete uri
        """

        if filter:
            filter = self.__make_query_filter(filter)

        if query:
            query = "&query=" + quote(query)

        if sort:
            sort = "&sort=" + quote(sort)

        if view:
            view = "&view=" + quote(view)

        if fields:
            fields = "&fields=" + quote(fields)

        if scope_uris:
            scope_uris = "&scopeUris=" + quote(scope_uris)

        path = uri if uri else self._uri
        self.__validate_resource_uri(path)

        symbol = '?' if '?' not in path else '&'

        uri = "{0}{1}start={2}&count={3}{4}{5}{6}{7}{8}{9}".format(path, symbol, start, count, filter, query, sort,
                                                                   view, fields, scope_uris)
        return uri

    def get_all(self, start=0, count=-1, filter='', query='', sort='', view='', fields='', uri=None, scope_uris=''):
        """
        Gets all items according with the given arguments.

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items (default).
            filter (list or str):
                A general filter/query string to narrow the list of items returned. The default is no
                filter; all resources are returned.
            query:
                A single query parameter can do what would take multiple parameters or multiple GET requests using
                filter. Use query for more complex queries. NOTE: This parameter is experimental for OneView 2.0.
            sort:
                The sort order of the returned data set. By default, the sort order is based on create time with the
                oldest entry first.
            view:
                Returns a specific subset of the attributes of the resource or collection by specifying the name of a
                predefined view. The default view is expand (show all attributes of the resource and all elements of
                the collections or resources).
            fields:
                Name of the fields.
            uri:
                A specific URI (optional)
            scope_uris:
                An expression to restrict the resources returned according to the scopes to
                which they are assigned.

        Returns:
            list: A list of items matching the specified filter.
        """

        uri = self.build_query_uri(start=start, count=count, filter=filter,
                                   query=query, sort=sort, view=view, fields=fields, uri=uri, scope_uris=scope_uris)

        logger.debug('Getting all resources with uri: {0}'.format(uri))

        result = self.__do_requests_to_getall(uri, count)

        return result

    def delete_all(self, filter, force=False, timeout=-1):
        """
        Deletes all resources from the appliance that match the provided filter.

        Args:
            filter:
                A general filter/query string to narrow the list of items deleted.
            force:
                If set to true, the operation completes despite any problems with network connectivity or errors
                on the resource itself. The default is false.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            bool: Indicates if the resources were successfully deleted.
        """
        uri = "{}?filter={}&force={}".format(self._uri, quote(filter), force)
        logger.debug("Delete all resources (uri = %s)" % uri)

        task, body = self._connection.delete(uri)

        if not task:
            # 204 NO CONTENT
            # Successful return from a synchronous delete operation.
            return True

        return self._task_monitor.wait_for_task(task, timeout=timeout)

    def delete(self, resource, force=False, timeout=-1, custom_headers=None):

        if not resource:
            logger.exception(RESOURCE_CLIENT_RESOURCE_WAS_NOT_PROVIDED)
            raise ValueError(RESOURCE_CLIENT_RESOURCE_WAS_NOT_PROVIDED)

        if isinstance(resource, dict):
            if 'uri' in resource and resource['uri']:
                uri = resource['uri']
            else:
                logger.exception(RESOURCE_CLIENT_UNKNOWN_OBJECT_TYPE)
                raise exceptions.HPEOneViewUnknownType(RESOURCE_CLIENT_UNKNOWN_OBJECT_TYPE)
        else:
            uri = self.build_uri(resource)

        if force:
            uri += '?force=True'

        logger.debug("Delete resource (uri = %s, resource = %s)" %
                     (self._uri, str(resource)))

        task, body = self._connection.delete(uri, custom_headers=custom_headers)

        if not task:
            # 204 NO CONTENT
            # Successful return from a synchronous delete operation.
            return True

        task = self._task_monitor.wait_for_task(task, timeout=timeout)

        return task

    def get_schema(self):
        logger.debug('Get schema (uri = %s, resource = %s)' %
                     (self._uri, self._uri))
        return self._connection.get(self._uri + '/schema')

    def get(self, id_or_uri):
        """
        Args:
            id_or_uri: Can be either the resource ID or the resource URI.

        Returns:
             The requested resource.
        """
        uri = self.build_uri(id_or_uri)
        logger.debug('Get resource (uri = %s, ID = %s)' %
                     (uri, str(id_or_uri)))
        return self._connection.get(uri)

    def get_collection(self, id_or_uri, filter=''):
        """
        Retrieves a collection of resources.

        Use this function when the 'start' and 'count' parameters are not allowed in the GET call.
        Otherwise, use get_all instead.

        Optional filtering criteria may be specified.

        Args:
            id_or_uri: Can be either the resource ID or the resource URI.
            filter (list or str): General filter/query string.

        Returns:
             Collection of the requested resource.
        """
        if filter:
            filter = self.__make_query_filter(filter)
            filter = "?" + filter[1:]

        uri = "{uri}{filter}".format(uri=self.build_uri(id_or_uri), filter=filter)
        logger.debug('Get resource collection (uri = %s)' % uri)
        response = self._connection.get(uri)
        return self.__get_members(response)

    def update_with_zero_body(self, uri, timeout=-1, custom_headers=None):
        """
        Makes a PUT request to update a resource when no request body is required.

        Args:
            uri:
                Can be either the resource ID or the resource URI.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers:
                Allows set specific HTTP headers.

        Returns:
            Updated resource.
        """
        logger.debug('Update with zero length body (uri = %s)' % uri)

        return self.__do_put(uri, None, timeout, custom_headers)

    def update(self, resource, uri=None, force=False, timeout=-1, custom_headers=None, default_values={}):
        """
        Makes a PUT request to update a resource when a request body is required.

        Args:
            resource:
                OneView resource dictionary.
            uri:
                Can be either the resource ID or the resource URI.
            force:
                If set to true, the operation completes despite any problems with network connectivity or errors
                on the resource itself. The default is false.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers:
                Allows set specific HTTP headers.
            default_values:
                Dictionary with default values grouped by OneView API version. This dictionary will be be merged with
                the resource dictionary only if the dictionary does not contain the keys.
                This argument is optional and the default value is an empty dictionary.
                Ex.:
                    default_values = {
                        '200': {"type": "logical-switch-group"},
                        '300': {"type": "logical-switch-groupV300"}
                    }

        Returns:
            Updated resource.
        """
        if not resource:
            logger.exception(RESOURCE_CLIENT_RESOURCE_WAS_NOT_PROVIDED)
            raise ValueError(RESOURCE_CLIENT_RESOURCE_WAS_NOT_PROVIDED)

        logger.debug('Update async (uri = %s, resource = %s)' %
                     (self._uri, str(resource)))

        if not uri:
            uri = resource['uri']

        if force:
            uri += '?force=True'

        resource = self.merge_default_values(resource, default_values)

        return self.__do_put(uri, resource, timeout, custom_headers)

    def create_with_zero_body(self, uri=None, timeout=-1, custom_headers=None):
        """
        Makes a POST request to create a resource when no request body is required.

        Args:
            uri:
                Can be either the resource ID or the resource URI.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers:
                Allows set specific HTTP headers.

        Returns:
            Created resource.
        """
        if not uri:
            uri = self._uri

        logger.debug('Create with zero body (uri = %s)' % uri)

        return self.__do_post(uri, {}, timeout, custom_headers)

    def create(self, resource, uri=None, timeout=-1, custom_headers=None, default_values={}):
        """
        Makes a POST request to create a resource when a request body is required.

        Args:
            resource:
                OneView resource dictionary.
            uri:
                Can be either the resource ID or the resource URI.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers:
                Allows set specific HTTP headers.
            default_values:
                Dictionary with default values grouped by OneView API version. This dictionary will be be merged with
                the resource dictionary only if the dictionary does not contain the keys.
                This argument is optional and the default value is an empty dictionary.
                Ex.:
                    default_values = {
                        '200': {"type": "logical-switch-group"},
                        '300': {"type": "logical-switch-groupV300"}
                    }

        Returns:
            Created resource.
        """
        if not resource:
            logger.exception(RESOURCE_CLIENT_RESOURCE_WAS_NOT_PROVIDED)
            raise ValueError(RESOURCE_CLIENT_RESOURCE_WAS_NOT_PROVIDED)

        if not uri:
            uri = self._uri

        logger.debug('Create (uri = %s, resource = %s)' %
                     (uri, str(resource)))

        resource = self.merge_default_values(resource, default_values)

        return self.__do_post(uri, resource, timeout, custom_headers)

    def upload(self, file_path, uri=None, timeout=-1):
        """
        Makes a multipart request.

        Args:
            file_path:
                File to upload.
            uri:
                A specific URI (optional).
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: Response body.
        """
        if not uri:
            uri = self._uri

        upload_file_name = os.path.basename(file_path)
        task, entity = self._connection.post_multipart_with_response_handling(uri, file_path, upload_file_name)

        if not task:
            return entity

        return self._task_monitor.wait_for_task(task, timeout)

    def patch(self, id_or_uri, operation, path, value, timeout=-1, custom_headers=None):
        """
        Uses the PATCH to update a resource.

        Only one operation can be performed in each PATCH call.

        Args:
            id_or_uri: Can be either the resource ID or the resource URI.
            operation: Patch operation
            path: Path
            value: Value
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            Updated resource.
        """
        patch_request_body = [{'op': operation, 'path': path, 'value': value}]

        return self.patch_request(id_or_uri=id_or_uri,
                                  body=patch_request_body,
                                  timeout=timeout,
                                  custom_headers=custom_headers)

    def patch_request(self, id_or_uri, body, timeout=-1, custom_headers=None):
        """
        Uses the PATCH to update a resource.

        Only one operation can be performed in each PATCH call.

        Args:
            id_or_uri: Can be either the resource ID or the resource URI.
            body: Patch request body
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            Updated resource.
        """
        uri = self.build_uri(id_or_uri)

        logger.debug('Patch resource (uri = %s, data = %s)' % (uri, body))

        custom_headers_copy = custom_headers.copy() if custom_headers else {}
        if self._connection._apiVersion >= 300 and 'Content-Type' not in custom_headers_copy:
            custom_headers_copy['Content-Type'] = 'application/json-patch+json'

        task, entity = self._connection.patch(uri, body, custom_headers=custom_headers_copy)

        if not task:
            return entity

        return self._task_monitor.wait_for_task(task, timeout)

    def get_by(self, field, value, uri=None):
        """
        This function uses get_all passing a filter.

        The search is case-insensitive.

        Args:
            field: Field name to filter.
            value: Value to filter.
            uri: Resource uri.

        Returns:
            dict
        """
        if not field:
            logger.exception(RESOURCE_CLIENT_INVALID_FIELD)
            raise ValueError(RESOURCE_CLIENT_INVALID_FIELD)

        if not uri:
            uri = self._uri
        self.__validate_resource_uri(uri)

        logger.debug('Get by (uri = %s, field = %s, value = %s)' %
                     (uri, field, str(value)))

        filter = "\"{0}='{1}'\"".format(field, value)
        results = self.get_all(filter=filter, uri=uri)

        # Workaround when the OneView filter does not work, it will filter again
        if "." not in field:
            # This filter only work for the first level
            results = [item for item in results if str(item.get(field, '')).lower() == value.lower()]

        return results

    def get_by_name(self, name):
        """
        Retrieve a resource by its name.

        Args:
            name: Resource name.

        Returns:
            dict
        """
        result = self.get_by('name', name)
        if not result:
            return None
        else:
            return result[0]

    def get_utilization(self, id_or_uri, fields=None, filter=None, refresh=False, view=None):
        """
        Retrieves historical utilization data for the specified resource, metrics, and time span.

        Args:
            id_or_uri:
                Resource identification
            fields:
                Name of the supported metric(s) to be retrieved in the format METRIC[,METRIC]...
                If unspecified, all metrics supported are returned.

            filter (list or str):
                Filters should be in the format FILTER_NAME=VALUE[,FILTER_NAME=VALUE]...
                E.g.: 'startDate=2016-05-30T11:20:44.541Z,endDate=2016-05-30T19:20:44.541Z'

                startDate
                    Start date of requested starting time range in ISO 8601 format. If omitted, the startDate is
                    determined by the endDate minus 24 hours.
                endDate
                    End date of requested starting time range in ISO 8601 format. When omitted, the endDate includes
                    the latest data sample available.

                If an excessive number of samples would otherwise be returned, the results will be segmented. The
                caller is responsible for comparing the returned sliceStartTime with the requested startTime in the
                response. If the sliceStartTime is greater than the oldestSampleTime and the requested start time,
                the caller is responsible for repeating the request with endTime set to sliceStartTime to obtain the
                next segment. This process is repeated until the full data set is retrieved.

                If the resource has no data, the UtilizationData is still returned but will contain no samples and
                sliceStartTime/sliceEndTime will be equal. oldestSampleTime/newestSampleTime will still be set
                appropriately (null if no data is available). If the filter does not happen to overlap the data
                that a resource has, then the metric history service will return null sample values for any
                missing samples.

            refresh:
                Specifies that if necessary, an additional request will be queued to obtain the most recent
                utilization data from the iLO. The response will not include any refreshed data. To track the
                availability of the newly collected data, monitor the TaskResource identified by the refreshTaskUri
                property in the response. If null, no refresh was queued.

            view:
                Specifies the resolution interval length of the samples to be retrieved. This is reflected in the
                resolution in the returned response. Utilization data is automatically purged to stay within storage
                space constraints. Supported views are listed below:

                native
                    Resolution of the samples returned will be one sample for each 5-minute time period. This is the
                    default view and matches the resolution of the data returned by the iLO. Samples at this resolution
                    are retained up to one year.
                hour
                    Resolution of the samples returned will be one sample for each 60-minute time period. Samples are
                    calculated by averaging the available 5-minute data samples that occurred within the hour, except
                    for PeakPower which is calculated by reporting the peak observed 5-minute sample value data during
                    the hour. Samples at this resolution are retained up to three years.
                day
                    Resolution of the samples returned will be one sample for each 24-hour time period. One day is a
                    24-hour period that starts at midnight GMT regardless of the time zone in which the appliance or
                    client is located. Samples are calculated by averaging the available 5-minute data samples that
                    occurred during the day, except for PeakPower which is calculated by reporting the peak observed
                    5-minute sample value data during the day. Samples at this resolution are retained up to three
                    years.

        Returns:
            dict
        """

        if not id_or_uri:
            raise ValueError(RESOURCE_CLIENT_INVALID_ID)

        query = ''

        if filter:
            query += self.__make_query_filter(filter)

        if fields:
            query += "&fields=" + quote(fields)

        if refresh:
            query += "&refresh=true"

        if view:
            query += "&view=" + quote(view)

        if query:
            query = "?" + query[1:]

        uri = "{0}/utilization{1}".format(self.build_uri(id_or_uri), query)

        return self._connection.get(uri)

    def create_report(self, uri, timeout=-1):
        """
        Creates a report and returns the output.

        Args:
            uri: URI
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            list:
        """
        logger.debug('Creating Report')
        task, _ = self._connection.post(uri, {})

        if not task:
            raise exceptions.HPEOneViewException(RESOURCE_CLIENT_TASK_EXPECTED)

        task = self._task_monitor.get_completed_task(task, timeout)

        return task['taskOutput']

    def build_uri(self, id_or_uri):
        if not id_or_uri:
            logger.exception(RESOURCE_CLIENT_INVALID_ID)
            raise ValueError(RESOURCE_CLIENT_INVALID_ID)

        if "/" in id_or_uri:
            self.__validate_resource_uri(id_or_uri)
            return id_or_uri
        else:
            return self._uri + "/" + id_or_uri

    def build_subresource_uri(self, resource_id_or_uri=None, subresource_id_or_uri=None, subresource_path=''):
        if subresource_id_or_uri and "/" in subresource_id_or_uri:
            return subresource_id_or_uri
        else:
            if not resource_id_or_uri:
                raise exceptions.HPEOneViewValueError(RESOURCE_ID_OR_URI_REQUIRED)

            resource_uri = self.build_uri(resource_id_or_uri)

            uri = "{}/{}/{}".format(resource_uri, subresource_path, str(subresource_id_or_uri or ''))
            uri = uri.replace("//", "/")

            if uri.endswith("/"):
                uri = uri[:-1]

            return uri

    def download(self, uri, file_path):
        """
        Downloads the contents of the requested URI to a stream.

        Args:
            uri: URI
            file_path: File path destination

        Returns:
            bool: Indicates if the file was successfully downloaded.
        """
        with open(file_path, 'wb') as file:
            return self._connection.download_to_stream(
                file,
                uri,
                custom_headers={"Accept": "application/octetstream;q=0.8, application/json"})

    def __validate_resource_uri(self, path):
        if self._uri not in path:
            logger.exception('Get by uri : unrecognized uri: (%s)' % path)
            raise exceptions.HPEOneViewUnknownType(UNRECOGNIZED_URI)

    def __make_query_filter(self, filters):
        if isinstance(filters, list):
            formated_filter = "&filter=".join(quote(f) for f in filters)
        else:
            formated_filter = quote(filters)

        return "&filter=" + formated_filter

    def __get_members(self, mlist):
        if mlist and 'members' in mlist and mlist['members']:
            return mlist['members']
        else:
            return []

    def __do_post(self, uri, resource, timeout, custom_headers):
        task, entity = self._connection.post(uri, resource, custom_headers=custom_headers)

        if not task:
            return entity

        return self._task_monitor.wait_for_task(task, timeout)

    def __do_put(self, uri, resource, timeout, custom_headers):
        task, body = self._connection.put(uri, resource, custom_headers=custom_headers)

        if not task:
            return body

        return self._task_monitor.wait_for_task(task, timeout)

    def __do_requests_to_getall(self, uri, requested_count):
        items = []

        while uri:
            logger.debug('Making HTTP request to get all resources. Uri: {0}'.format(uri))
            response = self._connection.get(uri)
            members = self.__get_members(response)
            items += members

            logger.debug("Response getAll: nextPageUri = {0}, members list length: {1}".format(uri, str(len(members))))
            uri = self.__get_next_page(response, items, requested_count)

        logger.debug('Total # of members found = {0}'.format(str(len(items))))
        return items

    def __get_next_page(self, response, items, requested_count):
        next_page_is_empty = response.get('nextPageUri') is None
        has_different_next_page = not response.get('uri') == response.get('nextPageUri')
        has_next_page = not next_page_is_empty and has_different_next_page

        if len(items) >= requested_count and requested_count != -1:
            return None

        return response.get('nextPageUri') if has_next_page else None

    def merge_default_values(self, resource, default_values):
        if not default_values:
            return resource

        merged_resource = None

        if not isinstance(resource, list):
            api_version = str(self._connection._apiVersion)
            data = default_values.get(api_version, {}).copy()
            merged_resource = merge_resources(data, resource)

        return merged_resource or resource


def merge_resources(resource1, resource2):
    """
    Updates a copy of resource1 with resource2 values and returns the merged dictionary.

    Args:
        resource1: original resource
        resource2: resource to update resource1

    Returns:
        dict: merged resource
    """
    merged = resource1.copy()
    merged.update(resource2)
    return merged


def merge_default_values(resource_list, default_values):
    """
    Generate a new list where each item of original resource_list will be merged with the default_values.

    Args:
        resource_list: list with items to be merged
        default_values: properties to be merged with each item list. If the item already contains some property
            the original value will be maintained.

    Returns:
        list: list containing each item merged with default_values
    """

    def merge_item(resource):
        return merge_resources(default_values, resource)

    return lmap(merge_item, resource_list)


def transform_list_to_dict(list):
    """
        Transforms a list into a dictionary, putting values as keys
    Args:
        id:
    Returns:
        dict: dictionary built
    """

    ret = {}

    for value in list:
        if isinstance(value, dict):
            ret.update(value)
        else:
            ret[str(value)] = True

    return ret


def extract_id_from_uri(id_or_uri):
    """
    Extract ID from the end of the URI

    Args:
        id_or_uri: ID or URI of the OneView resources.

    Returns:
        str: The string founded after the last "/"
    """
    if '/' in id_or_uri:
        return id_or_uri[id_or_uri.rindex('/') + 1:]
    else:
        return id_or_uri


def unavailable_method():
    """Raise an exception if method is not available for the resource."""
    raise exceptions.HPEOneViewUnavailableMethod(UNAVAILABLE_METHOD)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2021] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import Resource


class ApplianceDeviceSNMPv1TrapDestinations(Resource):
    """
    ApplianceDeviceSNMPv1TrapDestinations API client.
    The appliance has the ability to forward events received from monitored or managed
    server hardware to the specified destinations as SNMPv1 traps.
    """
    URI = '/rest/appliance/trap-destinations'

    def __init__(self, connection, data=None):
        super(ApplianceDeviceSNMPv1TrapDestinations, self).__init__(connection, data)

    def get_by_name(self, destination_address):
        """Retrieves a resource by its destination.

        Args:
            destination_address: Resource destination address.

        Returns:
            Resource object or None if resource does not exist.
        """
        return super(ApplianceDeviceSNMPv1TrapDestinations, self).get_by_field('destination', destination_address)

    def create_validation(self, destination, community_string, uri, timeout=-1):
        """
        Validate whether a hostname or ip address is a valid trap destination.
        If validation fails, it returns an error identifying the problem that occurred.

        Args:
            destination (str): destination ip address.
            community_string (str) - community string for the snmpv1 trap
            uri - uri of the snmpv1 trap destination
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView, just stop waiting for its completion.

        Return:
            Returns error message if destination already exists.

        """
        data = dict(
            destination=destination,
            communityString=community_string,
            uri=uri
        )
        validation_uri = "{}/validation".format(self.URI)
        return self._helper.create(data, uri=validation_uri, timeout=timeout)

    def create(self, data, id=None, timeout=-1):
        """
        Adds the specified trap forwarding destination.
        The trap destination associated with the specified id will be created if trap destination with that id does not exists.
        The id can only be an integer greater than 0.

        Args:
            data (dict): Object to create.
            id: id of the resource to be created
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView, just stop waiting for its completion.

        Return:
            dict: Created resource.

        """
        if not id:
            available_id = self.__get_first_available_id()
            uri = '{}/{}'.format(self.URI, available_id)
        else:
            uri = '{}/{}'.format(self.URI, id)

        self.create_validation(data['destination'], data['communityString'], uri)
        return super(ApplianceDeviceSNMPv1TrapDestinations, self).create(data, uri=uri, timeout=timeout)

    def __findFirstMissing(self, array, start, end):
        """
        Find the smallest elements missing in a sorted array.

        Args:
            array - list if ids
            start - starting index
            end - ending index

        Returns:
            int: The smallest element missing.
        """
        if (start > end):
            return end + 1

        if (start != array[start]):
            return start

        mid = int((start + end) / 2)

        if (array[mid] == mid):
            return self.__findFirstMissing(array, mid + 1, end)

        return self.__findFirstMissing(array, start, mid)

    def __get_first_available_id(self):
        """
        Private method to get the first available id.
        The id can only be an integer greater than 0.

        Returns:
            int: The first available id
        """
        traps = self._helper.get_all()
        if traps:
            used_ids = [0]
            for trap in traps:
                used_uris = trap.get('uri')
                used_ids.append(int(used_uris.split('/')[-1]))
            used_ids.sort()
            return self.__findFirstMissing(used_ids, 0, len(used_ids) - 1)
        else:
            return 1



# -*- coding: utf-8 -*-
###
# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import Resource, ensure_resource_client, unavailable_method


class SasLogicalInterconnects(Resource):
    """
    SAS Logical Interconnects API client.

    """
    URI = '/rest/sas-logical-interconnects'

    def __init__(self, connection, data=None):
        super(SasLogicalInterconnects, self).__init__(connection, data)

    def get_all(self, start=0, count=-1, fields='', filter='', query='', sort='', view=''):
        """
        Gets a list of SAS Logical Interconnects based on optional sorting and filtering and constrained by start and
        count parameters.

        Args:
            start:
                 The first item to return, using 0-based indexing. If not specified, the default is 0 - start with the
                 first available item.
            count:
                The number of resources to return. A count of -1 requests all items. The actual number of items in
                the response may differ from the requested count if the sum of start and count exceeds the total number
                of items.
            fields:
                 Specifies which fields should be returned in the result set.
            filter (list or str):
                 A general filter/query string to narrow the list of items returned. The default is no filter; all
                 resources are returned.
            query:
                 A general query string to narrow the list of resources returned. The default is no query (all
                 resources are returned).
            sort:
                The sort order of the returned data set. By default, the sort order is based on create time, with the
                oldest entry first.
            view:
                 Returns a specific subset of the attributes of the resource or collection, by specifying the name of a
                 predefined view. The default view is expand (show all attributes of the resource and all elements of
                 collections of resources).

        Returns:
            list: A list of SAS logical interconnects.
        """
        return self._helper.get_all(start=start, count=count, filter=filter, query=query, sort=sort, view=view,
                                    fields=fields)

    @ensure_resource_client
    def update_firmware(self, firmware_information, force=False):
        """
        Installs firmware to the member interconnects of a SAS Logical Interconnect.

        Args:
            firmware_information: Options to install firmware to a SAS Logical Interconnect.
            force: If sets to true, the operation completes despite any problems with the network connectivy
              or the erros on the resource itself.
        Returns:
            dict: SAS Logical Interconnect Firmware.
        """
        firmware_uri = "{}/firmware".format(self.data["uri"])
        result = self._helper.update(firmware_information, firmware_uri, force=force)
        self.refresh()

        return result

    @ensure_resource_client
    def get_firmware(self):
        """
        Gets baseline firmware information for a SAS Logical Interconnect.

        Returns:
            dict: SAS Logical Interconnect Firmware.
        """
        firmware_uri = "{}/firmware".format(self.data["uri"])
        return self._helper.do_get(firmware_uri)

    def update_compliance_all(self, information, timeout=-1):
        """
        Returns SAS Logical Interconnects to a consistent state. The current SAS Logical Interconnect state is
        compared to the associated SAS Logical Interconnect group.

        Args:
            information: Can be either the resource ID or URI.
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: SAS Logical Interconnect.
        """

        uri = self.URI + "/compliance"
        result = self._helper.update(information, uri, timeout=timeout)

        return result

    @ensure_resource_client
    def update_compliance(self, timeout=-1):
        """
        Returns a SAS Logical Interconnect to a consistent state. The current SAS Logical Interconnect state is
        compared to the associated SAS Logical Interconnect group.

        Args:
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: SAS Logical Interconnect.
        """
        uri = "{}/compliance".format(self.data["uri"])
        result = self._helper.update({}, uri, timeout=timeout)
        self.refresh()

        return result

    @ensure_resource_client
    def replace_drive_enclosure(self, information):
        """
        When a drive enclosure has been physically replaced, initiate the replacement operation that enables the
        new drive enclosure to take over as a replacement for the prior drive enclosure. The request requires
        specification of both the serial numbers of the original drive enclosure and its replacement to be provided.

        Args:
            information: Options to replace the drive enclosure.

        Returns:
            dict: SAS Logical Interconnect.
        """

        uri = "{}/replaceDriveEnclosure".format(self.data["uri"])
        result = self._helper.create(information, uri)
        self.refresh()

        return result

    @ensure_resource_client
    def update_configuration(self):
        """
        Asynchronously applies or re-applies the SAS Logical Interconnect configuration to all managed interconnects
        of a SAS Logical Interconnect.

        Returns:
            dict: SAS Logical Interconnect.
        """
        uri = "{}/configuration".format(self.data["uri"])
        result = self._helper.update({}, uri)
        self.refresh()

        return result

    def create(self):
        """Create method is not available"""
        unavailable_method()

    def delete(self):
        """Delete method is not available"""
        unavailable_method()

    def update(self):
        """update method is not available"""
        unavailable_method()



# -*- coding: utf-8 -*-
###
# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()


import logging
import time

from errno import ECONNABORTED, ETIMEDOUT, ENOEXEC, EINVAL, ENETUNREACH, ECONNRESET, ENETDOWN, ECONNREFUSED
from hpeOneView.exceptions import HPEOneViewInvalidResource, HPEOneViewTimeout, HPEOneViewTaskError, HPEOneViewUnknownType

TASK_PENDING_STATES = ['New', 'Starting', 'Pending', 'Running', 'Suspended', 'Stopping']
TASK_ERROR_STATES = ['Error', 'Warning', 'Terminated', 'Killed']
TASK_COMPLETED_STATES = ['Error', 'Warning', 'Completed', 'Terminated', 'Killed']

MSG_UNKNOWN_OBJECT_TYPE = 'Unknown object type'
MSG_TASK_TYPE_UNRECONIZED = "Task type: '%s' resource is not a recognized version"
MSG_UNKNOWN_EXCEPTION = 'Unknown Exception'
MSG_TIMEOUT = 'Waited %s seconds for task to complete, aborting'
MSG_INVALID_TASK = 'Invalid task was provided'

UNLIMITED_TIMEOUT = -1

logger = logging.getLogger(__name__)


class TaskMonitor(object):
    # Seconds to wait when a network failure occurs
    CONNECTION_FAILURE_TIMEOUT = 90

    # Known error numbers when the connection drops
    CONNECTION_FAILURE_ERROR_NUMBERS = [ENOEXEC, EINVAL, ENETUNREACH, ETIMEDOUT, ECONNRESET,
                                        ECONNABORTED, ENETUNREACH, ENETDOWN, ECONNREFUSED]

    def __init__(self, con):
        self._connection = con

    @staticmethod
    def get_current_seconds():
        return int(time.time())

    def wait_for_task(self, task, timeout=-1):
        """
        Wait for task execution and return associated resource.

        Args:
            task: task dict
            timeout: timeout in seconds

        Returns:
            Associated resource when creating or updating; True when deleting.
        """
        self.__wait_task_completion(task, timeout)

        task = self.get(task)

        logger.debug("Waiting for task. Percentage complete: " + str(task.get('computedPercentComplete')))
        logger.debug("Waiting for task. Task state: " + str(task.get('taskState')))

        task_response = self.__get_task_response(task)
        logger.debug('Task completed')
        return task_response

    def get_completed_task(self, task, timeout=-1):
        """
        Waits until the task is completed and returns the task resource.

        Args:
            task: TaskResource
            timeout: Timeout in seconds

        Returns:
            dict: TaskResource
        """
        self.__wait_task_completion(task, timeout)

        return self.get(task)

    def __wait_task_completion(self, task, timeout):
        if not task:
            raise HPEOneViewUnknownType(MSG_INVALID_TASK)

        logger.debug('Waiting for task completion...')

        # gets current cpu second for timeout
        start_time = self.get_current_seconds()
        connection_failure_control = dict(last_success=self.get_current_seconds())

        i = 0
        while self.is_task_running(task, connection_failure_control):
            # wait 1 to 10 seconds
            # the value increases to avoid flooding server with requests
            i = i + 1 if i < 10 else 10

            logger.debug("Waiting for task. Percentage complete: " + str(task.get('computedPercentComplete')))
            logger.debug("Waiting for task. Task state: " + str(task.get('taskState')))

            time.sleep(i)
            if (timeout != UNLIMITED_TIMEOUT) and (start_time + timeout < self.get_current_seconds()):
                raise HPEOneViewTimeout(MSG_TIMEOUT % str(timeout))

    def __get_task_response(self, task):
        deleted_states = ['Delete',
                          'Remove',
                          'Delete server hardware type',
                          'Remove SAN manager',
                          'Delete hypervisor cluster profile.']

        if task['taskState'] in TASK_ERROR_STATES and task['taskState'] != 'Warning':
            msg = None
            error_code = None
            if 'taskErrors' in task and len(task['taskErrors']) > 0:
                err = task['taskErrors'][0]
                if 'message' in err:
                    msg = err['message']

                error_code = err.get('errorCode')

            if msg:
                raise HPEOneViewTaskError(msg, error_code)
            elif 'taskStatus' in task and task['taskStatus']:
                raise HPEOneViewTaskError(task['taskStatus'], error_code)
            else:
                raise HPEOneViewTaskError(MSG_UNKNOWN_EXCEPTION, error_code)

        if 'name' in task and task['name'] in deleted_states:
            return True

        if 'type' in task and task['type'].startswith('Task'):
            # get associated resource when is not a delete task
            task, entity = self.get_associated_resource(task)
            return entity

        logger.warning('Task completed, unknown response: ' + str(task))
        return task

    def is_task_running(self, task, connection_failure_control=None):
        """
        Check if a task is running according to: TASK_PENDING_STATES ['New', 'Starting',
        'Pending', 'Running', 'Suspended', 'Stopping']

        Args:
            task (dict): OneView Task resource.
            connection_failure_control (dict):
                A dictionary instance that contains last_success for error tolerance control.

        Examples:

            >>> connection_failure_control = dict(last_success=int(time.time()))
            >>> while self.is_task_running(task, connection_failure_control):
            >>>     pass

        Returns:
            True when in TASK_PENDING_STATES; False when not.
        """
        if 'uri' in task:
            try:
                task = self.get(task)
                if connection_failure_control:
                    # Updates last success
                    connection_failure_control['last_success'] = self.get_current_seconds()
                if 'taskState' in task and task['taskState'] in TASK_PENDING_STATES:
                    return True

            except Exception as error:
                logger.error('; '.join(str(e) for e in error.args) + ' when waiting for the task: ' + str(task))

                if not connection_failure_control:
                    raise error

                if hasattr(error, 'errno') and error.errno in self.CONNECTION_FAILURE_ERROR_NUMBERS:
                    last_success = connection_failure_control['last_success']
                    if last_success + self.CONNECTION_FAILURE_TIMEOUT < self.get_current_seconds():
                        # Timeout reached
                        raise error
                    else:
                        # Return task is running when network instability occurs
                        return True
                else:
                    raise error

        return False

    def get(self, task):
        """
        Retrieve a task by its uri.

        Args:
            task: task dict, must have 'uri' key.

        Returns:
            task dict
        """

        task = self._connection.get(task['uri'])
        return task

    def get_associated_resource(self, task):
        """
        Retrieve a resource associated with a task.

        Args:
            task: task dict

        Returns:
            tuple: task (updated), the entity found (dict)
        """

        if not task:
            raise HPEOneViewUnknownType(MSG_INVALID_TASK)

        if task['category'] not in ['tasks', 'backups']:
            # it is an error if type is not in obj, so let the except flow
            raise HPEOneViewUnknownType(MSG_UNKNOWN_OBJECT_TYPE)

        if task['type'] in ['TaskResourceV2', 'TaskResourceV3']:
            resource_uri = task['associatedResource']['resourceUri']

            if resource_uri and resource_uri.startswith("/rest/appliance/support-dumps/"):
                # Specific for support dumps
                return task, resource_uri

        elif task['type'] == 'BACKUP':
            task = self._connection.get(task['taskUri'])
            resource_uri = task['uri']
        else:
            raise HPEOneViewInvalidResource(MSG_TASK_TYPE_UNRECONIZED % task['type'])

        entity = {}

        if resource_uri:
            entity = self._connection.get(resource_uri)

        return task, entity



# -*- coding: utf-8 -*-
###
# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()

from hpeOneView.resources.resource import ResourceClient


class Licenses(object):
    """
    Licenses. Gets a list of all license resources that are known by the appliance.

    """
    URI = '/rest/licenses'

    DEFAULT_VALUES = {
        '200': {'type': 'LicenseList'},
        '300': {"type": "LicenseList"},
        '500': {"type": "LicenseListV500"},
        '600': {"type": "LicenseListV500"}
    }

    def __init__(self, con):
        self._client = ResourceClient(con, self.URI)

    def create(self, resource, timeout=-1):
        """
        Add a license to the appliance.

        Args:
            resource (dict): Object to create.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView, just stop waiting for its completion.

        Returns:
            dict: Created resource.

        """
        return self._client.create(resource, timeout=timeout, default_values=self.DEFAULT_VALUES)

    def get_by_id(self, id_or_uri):
        """
        Gets the License with the specified ID.

        Args:
            id_or_uri: ID or URI of License.

        Returns:
            dict: The License.
        """
        return self._client.get(id_or_uri)

    def delete(self, id_or_uri):
        """
        Deletes a License.

        Args:
            resource: dict object to delete
            force:
                 If set to true, the operation completes despite any problems with
                 network connectivity or errors on the resource itself. The default is false.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            bool: Indicates if the resource was successfully deleted.

        """
        return self._client.delete(id_or_uri)

    def get_all(self, start=0, count=-1, filter='', sort=''):
        """
        Gets all the licenses loaded on the appliance. The collection is based on optional
        sorting and filtering and is constrained by start and count parameters.

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items.

                The actual number of items in the response might differ from the requested
                count if the sum of start and count exceeds the total number of items.
            filter (list or str):
                A general filter/query string to narrow the list of items returned. The
                default is no filter; all resources are returned.
            sort:
                The sort order of the returned data set. By default, the sort order is based
                on create time with the oldest entry first.

        Returns:
            list: A list of Licenses.
        """
        return self._client.get_all(start, count, filter=filter, sort=sort)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2022] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import Resource


class SanProviders(Resource):
    """
    SAN Managers API client.

    """

    URI = '/rest/fc-sans/providers'

    def __init__(self, connection, data=None):
        super(SanProviders, self).__init__(connection, data)

    def add(self, resource, provider_uri_or_id, timeout=-1):
        """
        Adds a Device Manager under the specified provider.

        Args:
            resource (dict): Object to add.
            provider_uri_or_id: ID or URI of provider.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView, just stop waiting for its completion.

        Returns:
            dict: Added SAN Manager.
        """
        uri = provider_uri_or_id + "/device-managers"

        return self._helper.create(data=resource, uri=uri, timeout=timeout)

    def get_provider_uri(self, provider_display_name):
        """
        Gets uri for a specific provider.

        Args:
            provider_display_name: Display name of the provider.

        Returns:
            uri
        """
        providers = self.get_by_field('displayName', provider_display_name)
        return providers.data['uri'] if providers else None

    def get_default_connection_info(self, provider_name):
        """
        Gets default connection info for a specific provider.

        Args:
            provider_name: Name of the provider.

        Returns:
            dict: Default connection information.
        """
        provider = self.get_by_field('displayName', provider_name)

        if provider:
            return provider.data['defaultConnectionInfo']
        else:
            return {}



# -*- coding: utf-8 -*-
###
# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import ResourceClient


class InterconnectLinkTopologies(object):
    """
    Interconnect Link Topologies API client.

    Note:
        This resource is only available on HPE Synergy.

    """

    URI = '/rest/interconnect-link-topologies'

    def __init__(self, con):
        self._connection = con
        self._client = ResourceClient(con, self.URI)

    def get_all(self, start=0, count=-1, filter='', sort=''):
        """
        Gets a paginated collection of all the interconnect link topologies based on the specified parameters.

        Filters can be used in the URL to control the number of interconnect link topologies that are returned.
        With no filters specified, the API returns all interconnect link toplogies.

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items.
                The actual number of items in the response might differ from the requested
                count if the sum of start and count exceeds the total number of items.
            filter (list or str):
                A general filter/query string to narrow the list of items returned. The
                default is no filter; all resources are returned.
            sort:
                The sort order of the returned data set. By default, the sort order is based
                on create time with the oldest entry first.

        Returns:
            list: A list of interconnect link topologies.

        """
        return self._client.get_all(start, count, filter=filter, sort=sort)

    def get(self, id_or_uri):
        """
        Gets an interconnect link topology by ID or by URI.

        Args:
            id_or_uri: Can be either the interconnect type id or the interconnect type uri.

        Returns:
            dict: The interconnect link topology.
        """
        return self._client.get(id_or_uri)

    def get_by(self, field, value):
        """
        Gets all interconnect link topologies that match the filter.
        The search is case-insensitive

        Args:
            field: Field name to filter.
            value: Value to filter.

        Returns:
            list: A list of interconnect link topologies.
        """
        return self._client.get_by(field, value)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2021] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()

from hpeOneView.resources.resource import Resource


class ApplianceNetworkInterfaces(Resource):
    """
    ApplianceNetworkInterface API Client.

    """
    URI = '/rest/appliance/network-interfaces'

    def __init__(self, connection, data=None):
        super(ApplianceNetworkInterfaces, self).__init__(connection, data)

    def get_all(self):
        return super(ApplianceNetworkInterfaces, self).get_by_uri(self.URI)

    def get_all_mac_address(self):
        """
        Gets unconfigured network interfaces on the appliance.
        Returns:
            dict: Unconfigured network interfaces on the appliance.
        """
        uri = self._helper.build_uri('mac-addresses')
        mac_addresses = super(ApplianceNetworkInterfaces, self).get_by_uri(uri)
        return mac_addresses.data['members']

    def get_by_mac_address(self, mac_address):
        """
        Gets the network interface by the macAddress.
        Returns:
            dict: Network interface with the given macAddress on the appliance.
        """

        resources = self.get_all().data["applianceNetworks"]
        for resource in resources:
            if resource['macAddress'].lower() == mac_address.lower():
                new_resource = self.new(self._connection, resource)
            else:
                new_resource = None
        return new_resource



# -*- coding: utf-8 -*-
###
# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from future import standard_library

standard_library.install_aliases()

from hpeOneView.resources.resource import (Resource, ResourcePatchMixin)


class FcNetworks(ResourcePatchMixin, Resource):
    """
    Fibre Channel networks API client.

    """

    URI = '/rest/fc-networks'

    DEFAULT_VALUES = {
        '200': {'type': 'fc-networkV2'},
        '300': {"type": "fc-networkV300"},
        '500': {"type": "fc-networkV300"},
        '600': {"type": "fc-networkV4"},
        '800': {"type": "fc-networkV4"},
        '1000': {"type": "fc-networkV4"},
        '1200': {"type": "fc-networkV4"},
        '1600': {"type": "fc-networkV4"},
        '1800': {"type": "fc-networkV4"}
    }

    def __init__(self, connection, data=None):
        super(FcNetworks, self).__init__(connection, data)

    def delete_bulk(self, resource, timeout=-1):
        """
        Deletes bulk fc networks.

        Args:
            resource (dict): Specifications to delete in bulk.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        """
        uri = self.URI + '/bulk-delete'

        return self._helper.create(resource, uri=uri, timeout=timeout)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()

from hpeOneView.resources.resource import Resource


class Restores(Resource):
    """
    Restore API client for initiate a restore of an appliance and to get the status of the restore operation.
    """
    URI = '/rest/restores'

    DEFAULT_VALUES = {
        '800': {"type": "RESTORE"},
        '1000': {"type": "RESTOREV1000"},
        '1200': {"type": "RESTOREV1000"},
        '1600': {"type": "RESTOREV1000"}
    }

    def __init__(self, connection, data=None):
        super(Restores, self).__init__(connection, data)

    def get_failure(self):
        """
        Retrieves the result of an appliance restore operation after it has completed.

        The restore result remains valid until a user logs in. After a user logs in, the restore result will be reset.
        This rest request will return only the valid result after restore has completed and before a user logs in.

        Returns:
            dict: Restore Result
        """
        failure_uri = "{0}/failure".format(self.URI)
        return self._helper.do_get(failure_uri)

    def restore(self, resource, timeout=-1):
        """
        Starts a restore operation with the specified backup file. The backup must be uploaded to the appliance
        prior to running this command. Only one restore can run at a time.

        Args:
            resource (dict): Config to restore.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView, just stop waiting for its completion.

        Returns:
            dict: Restore.

        """
        return super(Restores, self).create(resource, timeout=timeout)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import ResourceClient


class LogicalSwitches(object):
    """
    Logical Switches API client.

    Note:
        This resource is only available on C7000 enclosures.

    """
    URI = '/rest/logical-switches'

    SWITCH_DEFAULT_VALUES = {
        '200': {"type": "logical-switch"},
        '300': {"type": "logical-switchV300"},
        '500': {"type": "logical-switchV300"},
        '600': {"type": "logical-switchV4"}
    }

    def __init__(self, con):
        self._connection = con
        self._client = ResourceClient(con, self.URI)

    def get_all(self, start=0, count=-1, filter='', sort=''):
        """
        Gets a paginated collection of Logical Switches. The collection is based on optional
        sorting and filtering and is constrained by start and count parameters.

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items.

                The actual number of items in the response might differ from the requested
                count if the sum of start and count exceeds the total number of items.
            filter (list or str):
                A general filter/query string to narrow the list of items returned. The
                default is no filter; all resources are returned.
            sort:
                The sort order of the returned data set. By default, the sort order is based
                on create time with the oldest entry first.

        Returns:
            list: A list of Logical Switches.
        """
        return self._client.get_all(start, count, filter=filter, sort=sort)

    def delete(self, resource, force=False, timeout=-1):
        """
        Deletes a Logical Switch.

        Args:
            resource: dict object to delete
            force:
                 If set to true, the operation completes despite any problems with
                 network connectivity or errors on the resource itself. The default is false.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            bool: Indicates if the resource was successfully deleted.

        """
        return self._client.delete(resource, force=force, timeout=timeout)

    def get(self, id_or_uri):
        """
        Gets the Logical Switch with the specified ID.

        Args:
            id_or_uri: Can be either the Logical Switch ID or URI

        Returns:
            dict: Logical Switch.
        """
        return self._client.get(id_or_uri)

    def create(self, resource, timeout=-1):
        """
        Creates a Logical Switch.

        Args:
            resource (dict): Object to create.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView, just stop waiting for its completion.

        Returns:
            dict: Created resource.
        """
        self.__set_default_values(resource)
        return self._client.create(resource, timeout=timeout)

    def update(self, resource, timeout=-1):
        """
        Updates a Logical Switch.

        Args:
            resource (dict): Object to update.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView, just stop waiting for its completion.

        Returns:
            dict: Updated resource.
        """
        self.__set_default_values(resource)
        uri = self._client.build_uri(resource['logicalSwitch']['uri'])
        return self._client.update(resource, uri=uri, timeout=timeout)

    def get_by(self, field, value):
        """
        Gets all Logical Switches that match the filter.

        The search is case-insensitive.

        Args:
            field: Field name to filter.
            value: Value to filter.

        Returns:
            list: A list of Logical Switches.
        """
        return self._client.get_by(field, value)

    def refresh(self, id_or_uri, timeout=-1):
        """
        The Refresh action reclaims the top-of-rack switches in a logical switch.

        Args:
            id_or_uri:
                Can be either the Logical Switch ID or URI
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView, just stop waiting for its completion.

        Returns:
            dict: The Logical Switch
        """
        uri = self._client.build_uri(id_or_uri) + "/refresh"
        return self._client.update_with_zero_body(uri, timeout=timeout)

    def patch(self, id_or_uri, operation, path, value, timeout=-1):
        """
        Uses the PATCH to update a resource for a given logical switch group.

        Only one operation can be performed in each PATCH call.

        Args:
            id_or_uri: Can be either the resource ID or the resource URI.
            operation: Patch operation
            path: Path
            value: Value
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: Updated resource.
        """
        return self._client.patch(id_or_uri, operation, path, value, timeout=timeout)

    def __set_default_values(self, resource):
        if 'logicalSwitch' in resource:
            resource['logicalSwitch'] = self._client.merge_default_values(resource['logicalSwitch'],
                                                                          self.SWITCH_DEFAULT_VALUES)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import Resource, ResourceSchemaMixin


class FirmwareDrivers(ResourceSchemaMixin, Resource):
    """
    The firmware drivers resource managers provides REST APIs to retrieve the firmware bundle inventory data.
    Note: As per the API docs, 'type' field is not present in the firmware drivers POST call. So default values are not added.
    """
    URI = '/rest/firmware-drivers'

    def __init__(self, connection, data=None):
        super(FirmwareDrivers, self).__init__(connection, data)

    def get_by_name(self, name, version=None):
        """
        Retrieves the specified firmware driver resource by name and version.

        Args:
            name: name of specified firmware driver resource
            version: version of specified firmware driver resource (optional)

        Return:
            dict: Get response of specified firmware driver resource.
        """
        results = super(FirmwareDrivers, self).get_all()

        # filter by name and version(optional)
        result = [item for item in results if item.get('name', "").lower() == name.lower()]
        if version:
            result = [item for item in result if str(item.get('version', "")).lower() == version.lower()]

        if result:
            data = result[0]
            new_resource = self.new(self._connection, data)
        else:
            new_resource = None
        return new_resource



# -*- coding: utf-8 -*-
###
# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()

from hpeOneView.resources.resource import ResourceClient


class MetricStreaming(object):
    """
    Metrics API client.

    Metrics can be relayed from OneView for managed resources at a specified interval. The following steps can be
    followed to enable the metric relay in OneView:

        * Get the list of resource types and metrics which can be configured for live streaming
        * Configure the live metric stream in OneView
        * Receive the stream of metric on MSMB

    The list below describes the structure of message relayed to MSMB:
        startTime (str):
            The starting time of the metric collection.
        sampleIntervalInSeconds (int):
            Interval between samples.
        numberOfSamples (int):
            Number of samples in the list for each metric type.
        resourceType (str):
            Identifies the resource type.
        resourceDataList (list):
            Metric sample list.
        uri (str):
            Canonical URI of the resource.
        category (str):
            Identifies the category of resource. The supported devices are server-hardware, enclosures, and
            power-devices.
        created (timestamp):
            Date and time when the resource was created.
        modified (timestamp):
            Date and time when the resource was last modified.
        eTag (str):
            Entity tag/version ID of the resource, the same value that is returned in the ETag header on a GET of the
            resource.
        type (str):
            Uniquely identifies the type of the JSON object.

    """
    URI = '/rest/metrics'

    def __init__(self, con):
        self._connection = con
        self._client = ResourceClient(con, self.URI)

    def get_capability(self):
        """
        Fetches the list of resource types and supported metrics that OneView is capable of relaying.

        Returns:
            list: List of resource types and supported metrics.
        """
        return self._client.get(self.URI + "/capability")

    def get_configuration(self):
        """
        Fetches the current configuration for which metrics are being relayed.

        Returns:
            list: List of objects which contain frequency, sample interval, and source type for each resource-type.

        """
        return self._client.get(self.URI + "/configuration")

    def update_configuration(self, configuration):
        """
        Updates the metrics configuration with the new values. Overwrites the existing configuration.

        Args:
            configuration (dict):
                Dictionary with a list of objects which contain frequency, sample interval, and source type for each
                resource-type.

        Returns:
            dict: The current configuration for which metrics are being relayed.

        """
        return self._client.update(configuration, uri=self.URI + "/configuration")



# -*- coding: utf-8 -*-
###
# (C) Copyright [2021] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from future import standard_library

standard_library.install_aliases()

from hpeOneView.resources.resource import Resource, ResourceSchemaMixin


class IdPools(Resource, ResourceSchemaMixin):
    """
    Class for Id Pools API client.
    """
    URI = '/rest/id-pools'

    def __init__(self, connection, data=None):
        super(IdPools, self).__init__(connection, data)

    def get_pool_type(self, pool_type):
        """
        Gets a pool along with the list of ranges present in it

        Args:
          pool_type: Id pool type

        Returns:
          dict: List of ranges
        """
        uri = self._helper.build_uri(pool_type)
        return super(IdPools, self).get_by_uri(uri)

    def update_pool_type(self, data, pool_type, timeout=-1):
        """
        Enables or disables the pool

        Args:
          data: List of ID ranges
          pool_type: Id pool type

        Returns:
            dict: Updated Resource.
        """
        uri = self._helper.build_uri(pool_type)
        return self._helper.update(data, uri, timeout=timeout)

    def validate_id_pool(self, pool_type, ids_pools):
        """
        Validates an ID pool.

        Args:
            pool_type: Id pool type

            ids_pools (list):
                List of Id Pools.

        Returns:
            dict: A dict containing a list with IDs.
        """
        uri = self._helper.build_uri(pool_type) + "/validate?idList=" + "&idList=".join(ids_pools)
        return super(IdPools, self).get_by_uri(uri)

    def validate(self, information, pool_type, timeout=-1):
        """
        Validates a set of user specified IDs to reserve in the pool.

        This API can be used to check if the specified IDs can be allocated.

        Args:
            information (dict):
                Information to update. Can result in system specified IDs or the system reserving user-specified IDs.

            pool_type: Id pool type

            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: A dict containing a list with IDs.
        """
        uri = self._helper.build_uri(pool_type) + "/validate"
        return self._helper.update(information, uri, timeout=timeout)

    def allocate(self, information, pool_type, timeout=-1):
        """
        Allocates a set of IDs from range.

        The allocator returned contains the list of IDs successfully allocated.

        Args:
            information (dict):
                Information to update. Can result in system specified IDs or the system reserving user-specified IDs.

            pool_type: Id pool type

            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: A dict containing a list with IDs.
        """
        uri = self._helper.build_uri(pool_type) + "/allocator"

        return self._helper.update(information, uri, timeout=timeout)

    def collect(self, information, pool_type, timeout=-1):
        """
        Collects one or more IDs to be returned to a pool.

        Args:
            information (dict):
                The list of IDs to be collected

            pool_type: Id pool type

            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Returns:
            dict: Collector containing list of collected IDs successfully collected.
        """
        uri = self._helper.build_uri(pool_type) + "/collector"

        return self._helper.update(information, uri, timeout=timeout)

    def get_check_range_availability(self, pool_type, ids_pools):
        """
        Checks the range availability in the ID pool.

        Args:
            pool_type: Id pool type

            ids_pools (list):
                List of Id Pools.

        Returns:
            dict: A dict containing a list with IDs.
        """
        uri = self._helper.build_uri(pool_type) + "/checkrangeavailability?idList=" + "&idList=".join(ids_pools)
        return super(IdPools, self).get_by_uri(uri)

    def generate(self, pool_type):
        """
        Generates and returns a random range.

        Args:
            pool_type: Id pool type

        Returns:
            dict: A dict containing a list with IDs.
        """
        uri = self._helper.build_uri(pool_type) + "/generate"
        return super(IdPools, self).get_by_uri(uri)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
from future import standard_library

standard_library.install_aliases()


from hpeOneView.resources.resource import Resource


class HypervisorClusterProfiles(Resource):
    """
    The cluster profile resource manager REST APIs to create, retrieve, modify and delete hypervisor cluster profiles.
    """
    URI = '/rest/hypervisor-cluster-profiles'
    DEFAULT_VALUES = {
        '2200': {'type': 'HypervisorClusterProfileV4'}
    }

    def __init__(self, connection, data=None):
        super(HypervisorClusterProfiles, self).__init__(connection, data)
        self.__default_values = {
            'type': 'HypervisorClusterProfileV3'
        }

    def get_all(self, start=0, count=-1, filter='', sort='', query='', scope_uris=''):
        """
        Gets a list of hypervisor cluster profiles based on optional sorting and filtering,
        and constrained by start and count parameters.

        The maximum number of profiles is restricted to 100, i.e.,
        if user requests more than 100, this will be internally limited to 100.

        Args:
            start:
                The first item to return, using 0-based indexing.
                If not specified, the default is 0 - start with the first available item.
            count:
                The number of resources to return. A count of -1 requests all items.
                The actual number of items in the response might differ from the requested
                count if the sum of start and count exceeds the total number of items.
            filter (list or str):
                A general filter/query string to narrow the list of items returned. The
                default is no filter; all resources are returned.
            sort:
                The sort order of the returned data set. By default, the sort order is based
                on create time with the oldest entry first.
            query:
                 A general query string to narrow the list of resources returned. The default
                 is no query - all resources are returned.
            scope_uris:
                An expression to restrict the resources returned according to the scopes to
                which they are assigned.

        Return:
             list: List of Hypervisor cluster profiles
        """
        return self._helper.get_all(start, count, filter=filter, sort=sort, query=query, scope_uris=scope_uris)

    def create_virtual_switch_layout(self, data=None, timeout=-1):
        """Generates vSwitch layout using information specified in the request body.

        Args:
            data: Fields passed to create the resource.
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.

        Return:
            Created virtual switch layout.
        """
        if not data:
            data = {}

        vswitch_uri = "{}/virtualswitch-layout".format(self.URI)
        return self._helper.create(data, vswitch_uri, timeout)

    def get_compliance_preview(self):
        """
        Gets the preview of manual and automatic updates required to make the cluster profile consistent with its template.

        Return:
            cluster profile compliance preview
        """
        compliance_uri = "{0}/compliance-preview".format(self.data["uri"])
        return self._helper.do_get(compliance_uri)

    def delete(self, timeout=-1, soft_delete=False, force=False):
        """
        Deletes a hypervisor cluster profile object from the appliance based on Hypervisor Cluster Profile UUID

        Args:
            force:
                If set to true, the operation completes despite any probles with network
                connectivity or errors on the resource itself. Default is false.
            soft_delete:
                If set to true, the hypervisor cluster profile and its hypervisor profiles
                are removed from appliance only. If set to false, the associated cluster and
                hosts are also removed in the hypervisor manager.

        Return:
            Boolean value. True for success and False for failure.
        """
        uri = "{}?softDelete={}".format(self.data['uri'], soft_delete)

        if force:
            uri += '&force=True'

        return self._helper.delete(uri, timeout=timeout)



# -*- coding: utf-8 -*-
###
# (C) Copyright [2021] Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from future import standard_library
from urllib.parse import quote
from copy import deepcopy

standard_library.install_aliases()


from hpeOneView.resources.resource import Resource
from hpeOneView.exceptions import HPEOneViewException


class Users(Resource):
    """
    Users API client.

    """

    URI = '/rest/users'

    def __init__(self, connection, data=None):
        super(Users, self).__init__(connection, data)
        self.__default_values = {
            'type': 'UserAndRoles'
        }

    def validate_user_name(self, user_name, timeout=-1):
        """
        Verifies if a userName is already in use.

        Args:
            user_name (str):
                The userName to be verified.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation in
                OneView, just stops waiting for its completion.

        Returns: True if user name is in use, False if it is not.
        """
        uri = self.URI + '/validateLoginName/' + user_name
        return self.create(uri=uri)

    def validate_full_name(self, full_name, timeout=-1):
        """
        Verifies if a fullName is already in use.

        Args:
            full_name (str):
                The fullName to be verified.
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation in
                OneView, just stops waiting for its completion.

        Returns: True if full name is in use, False if it is not.
        """
        uri = self.URI + '/validateUserName/' + full_name
        return self.create(uri=uri)

    def change_password(self, resource):
        """
        Change one's own password

        Args:
            resource (dict): Object to change password
            timeout:
                Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation in
                OneView, just stops waiting for its completion.

        """
        uri = self._helper.build_uri('changePassword')
        return self._helper.create(resource, uri)

    def get_role_associated_with_userName(self, userName):
        """
        Gets a user by userName.

        Args:
            name (str): userName of the user.

        Returns:
            dict object: User
        """

        users = self.get_all()

        result = [x for x in users if x['userName'] == userName]
        resource = result[0] if result else None
        if resource:
            uri = self.URI + '/role/' + userName
            data = self.get_by_uri(uri).data
            result = data['members']
            return result
        else:
            return None

    def get_by_userName(self, name):
        """
        Gets a complete json body for username

        Args:
          name (str): userName of the user

        Returns:
           dict object: User
        """

        uri = self._helper.build_uri(name)
        try:
            resource = self.get_by_uri(uri)
        except HPEOneViewException:
            resource = None

        return resource

    def get_user_by_role(self, rolename):
        """
        Gets all the users associated with this role

        Args:
          rolename (str): rolename of the user

        Returns:
          list: User
        """

        rolename = quote(rolename)
        uri = self.URI + '/roles/users/' + rolename
        data = self.get_by_uri(uri).data
        result = []
        for i in range(0, len(data['members'])):
            result.append(data["members"][i])

        return result

    def create_multiple_user(self, data):
        """
        Create a multiple user

        Agrs:
          data (list): multiple user

        Returns:
          dict object: User
        """

        uri = self.URI + '?multiResource=true'
        return self.create(data, uri)

    def update(self, data=None, timeout=-1, custom_headers=None, force=False):
        """
        Makes a PUT request to update a resource when a request body is required.

        Args:
            data (dict): Data to update the resource.
            timeout: Timeout in seconds. Wait for task completion by default. The timeout does not abort the operation
                in OneView; it just stops waiting for its completion.
            custom_headers: Allows to add custom HTTP headers.
            force: Force the update operation.

        Returns:
            A dict with the updated resource data.
        """

        resource = deepcopy(self.data)
        resource.update(data)

        self.data = self._helper.update(resource, self.URI, force, timeout, custom_headers)

        return self

    def add_role_to_userName(self, username, data):
        """
        Add roles to a given user name

        Args:
          username (str): userName of the user
          data (list): roles to be added

        Returns:
          dict object: User
        """

        uri = self.URI + '/' + username + '/roles?multiResource=true'
        return self.create(data, uri)

    def update_role_to_userName(self, username, data):
        """
        Update roles to a given user name

        Agrs:
          username (str): username of the user
          data (list): roles to be updated

        Return:
          dict: User
        """

        uri = self.URI + '/' + username + '/roles?multiResource=true'
        return self._helper.update(data, uri)

    def remove_role_from_username(self, username, data):
        """
        Removes a specified role from the username

        Args:
          username (str): username of the user
          data (str/list): list role to be removed from user

        Return:
          boolean
        """

        rolelist_query = self.query_filter(data)
        uri = self.URI + '/roles?filter' + '="userName=\'{}\'{}'.format(username, rolelist_query)
        return self._helper.delete(uri)

    def delete_multiple_user(self, data):
        """
        Delete the multiple users

        Args:
          data (list): List of users to be deleted

        Returns:
          None
        """

        uri = self.URI + '?query='

        for i in range(0, len(data)):
            uri = uri + '(loginname=\'{}\')'.format(data[i])
            if i == len(data) - 1:
                break
            uri = uri + quote(' or ')
        self._helper.delete(uri)

    def query_filter(self, filters):

        formated_filter = ''
        base_query = "\"&filter=\"roleName=\'{}\'\""
        if isinstance(filters, list):
            for role in filters:
                formated_filter += base_query.format(quote(role))
            return formated_filter

        if isinstance(filters, str):
            return base_query.format(quote(filters))
